########################################################################
#
# File Name:            ReaderBase.py
#
# Documentation:        http://docs.ftsuite.com/4Suite/Lib/ReaderBase.py.html
#
"""
The base class for all readers in 4Suite
WWW: http://4suite.com/4Suite         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc., USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""
#"
import string, urllib, cStringIO, os
from Ft.Lib import Uri

try:
    import codecs
    from types import UnicodeType
    encoder = codecs.lookup("utf-8")[0] # encode,decode,reader,writer
    def StrStream(st):
        if type(st) is UnicodeType:
            st = encoder(st)[0]
        return cStringIO.StringIO(st)
except ImportError:
    StrStream = lambda x: cStringIO.StringIO(x)

#Doesn't really belong there
from xml.dom.ext.reader import Reader


class DomletteReader:
    def __init__(self, force8Bit=0):
        self.uriResolver = Uri.BASIC_RESOLVER
        self._force8Bit = force8Bit
        return

    def clone(self):
        """Used to create a new copy of this instance"""
        if hasattr(self,'__getinitargs__'):
            return apply(self.__class__,self.__getinitargs__())
        else:
            return self.__class__()

    def fromStream(self, stream, refUri='', ownerDoc=None,
                   stripElements=None):
        "Create a DOM from a stream"
        raise Exception("NOT OVERIDDEN")

    def releaseNode(self, dom):
        "Free a DOM tree"
        raise Exception("NOT OVERIDDEN")

    def fromString(self, st, refUri='', ownerDoc=None, stripElements=None):
        "Create a DOM from a string"
        #The default implementaton
        self._baseUri = refUri
        stream = StrStream(st)
        rt = self.fromStream(stream, refUri, ownerDoc, stripElements)
        stream.close()
        return rt

    def fromUri(self, uri, baseUri='', ownerDoc=None, stripElements=None):
        "Create a DOM from a URI"
        stream = self.uriResolver.resolve(uri, baseUri)
        uri = self.uriResolver.normalize(uri, baseUri)
        newBaseUri = urllib.basejoin(baseUri, uri) 
        rt = self.fromStream(stream, newBaseUri, ownerDoc, stripElements) 
        stream.close()
        return rt


def AssertReader(reader):
    if not isinstance(reader, Reader) and not isinstance(reader, DomletteReader):
        raise "Not a Reader %s" % str(reader)

