########################################################################
#
# File Name:            TreeCompare.py
#
# Documentation:        http://docs.4suite.com/Lib/TreeCompare.py.html
#
"""
Tools to compare two XML strings.  Mostly for test suites
WWW: http://4suite.com/4DOM         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""
import re
from xml.dom.ext.reader import PyExpat
from xml.dom import Node
from xml.dom.ext.reader import HtmlLib

g_htmlTest = re.compile("(<!doctype html)|(<html>)",re.IGNORECASE)
g_xmlTest = re.compile(r'<\?xml')

def TreeCompare(xml1, xml2):
    #See if we need to use XML or HTML
    if g_htmlTest.search(xml1) and not g_xmlTest.match(xml1):
        reader = HtmlLib.Reader()
    else:
        reader = PyExpat.Reader()
    try:
        doc1 = reader.fromString(xml1)
    except:
        print xml1
        raise
    try:
        doc2 = reader.fromString(xml2)
    except:
        print xml2
        raise
    result = NodeCompare(doc1, doc2)
    reader.releaseNode(doc1)
    reader.releaseNode(doc2)
    return not result


def NodeCompare(node1, node2):
    if node1.nodeType != node2.nodeType:
        return 0
    if node1.nodeType == Node.DOCUMENT_NODE:
        return NodeCompare(node1.documentElement, node2.documentElement)
    elif node1.nodeType == Node.ELEMENT_NODE:
        if (node1.namespaceURI, node1.localName) != (node2.namespaceURI, node2.localName):
            return 0
        if len(node1.attributes.keys()) != len(node2.attributes.keys()):
            return 0
        for attr in node1.attributes:
            if node2.hasAttributeNS(attr.namespaceURI, attr.localName):
                if not node2.getAttributeNS(attr.namespaceURI, attr.localName) == attr.value:
                    return 0
            elif not attr.namespaceURI and node2.hasAttribute(attr.name):
                if not node2.getAttribute(attr.name) == attr.value:
                    return 0
            else:
                return 0
        if len(node1.childNodes) != len(node2.childNodes):
            return 0
        for child1, child2 in map(lambda x, y: (x, y), node1.childNodes, node2.childNodes):
            if not NodeCompare(child1, child2):
                return 0
        return 1
    elif node1.nodeType == Node.TEXT_NODE:
        return node1.data == node2.data
    else:
        return 1
        
