########################################################################
#
# File Name:            XInclude.py
#
# Documentation:        http://docs.ftsuite.com/Lib/XInclude.py.html
#
"""
A Utility to expand XInclude
WWW: http://4suite.com/XInclude         e-mail: support@4suite.com

Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""
#'
import os,urllib
import xml.dom
from Ft import XPointer
from Ft.Lib import Uri
from xml import xpath
from xml.xpath import Context
g_includeExpr = xpath.Compile("//*[@xinclude:href]")

XINCLUDE_NAMESPACE = 'http://www.w3.org/1999/XML/xinclude'

class XIncludeException(Exception):

    CIRCULAR_INCLUDE_ERROR = 1
    RESOURCE_ERROR = 2

    def __init__(self, code, extra=None):
        # args ::= (code, object?)?
        msg = g_errorMessages[code]
        if extra:
            msg = msg + '\n' + str(extra)
        Exception.__init__(self, msg)
        self.code = code
        self.msg = msg

g_errorMessages = {
    XIncludeException.CIRCULAR_INCLUDE_ERROR: "Circular Inclusion",
    XIncludeException.RESOURCE_ERROR: "Invalid resource, or not well-formed XML",
    }


def ProcessIncludesFromUri(uri, reader=None, visited=None, baseUri='',
                           uriResolver=None):
    reader = reader or PyExpatReader()
    if uriResolver:
        reader.uriResolver = uriResolver
    node = reader.fromUri(uri, baseUri=baseUri)
    return ProcessIncludesFromNode(node, reader, visited, baseUri)


def ProcessIncludesFromString(st, reader=None, visited=None, baseUri = ''):
    reader = reader or PyExpatReader()
    node = reader.fromString(st)
    return ProcessIncludesFromNode(node, reader, visited, baseUri)


def ProcessIncludesFromNode(node, reader=None, visited=None, baseUri = ''):
    nss={'xinclude':XINCLUDE_NAMESPACE}
    reader = reader or PyExpatReader()
    con = Context.Context(node,processorNss=nss)
    nodes = g_includeExpr.evaluate(con)
    visited = visited or []
    for n in nodes:
        _ProcessNode(n, visited, reader, baseUri)
    return node

def _ProcessNode(node, visited, reader, baseUri):    
    href = node.getAttributeNS(XINCLUDE_NAMESPACE,'href')
    if href in visited:
        raise XIncludeException(XIncludeException.CIRCULAR_INCLUDE_ERROR)
    parse = node.getAttributeNS(XINCLUDE_NAMESPACE,'parse')
    if not parse or parse=='xml':
        #Use XPointer to load the new
        #FIXME catch parse exception
        replaceNode = XPointer.SelectUri(href, reader=reader, base=baseUri)
        replaceNode = ProcessIncludesFromNode(replaceNode, visited=visited,
                                              reader=reader)
        if replaceNode.nodeType == xml.dom.Node.DOCUMENT_NODE:
            replaceNode = replaceNode.documentElement
        visited.append(href)
    else:
        if os.path.exists(href):
            st = open(href)
        else:
            st = urllib.open(href)
        replaceNode = node.ownerDocument.createTextNode(st.read())
        st.close()

    node.parentNode.replaceChild(replaceNode,node)
    
from Ft.Lib.pDomlette import PyExpatReader
