########################################################################
#
# File Name:            pDomletteReader.py
#
# Documentation:        http://docs.4suite.org/pDomletteReader.py.html
#
"""
Reader tools for pDomlette
WWW: http://4suite.org        e-mail: support@4suite.org

Copyright (c) 2001 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import sys, string, os, urllib
from types import StringType
stringTypes = [StringType]
try:
    from types import UnicodeType
    stringTypes.append(UnicodeType)
except ImportError, e:
    pass

from xml.sax import saxlib, sax2exts, handler, xmlreader
from Ft.Lib import ReaderBase, FtException, Error
from xml.dom.ext.reader.Sax2 import XmlDomGenerator
from xml.parsers import expat
from xml.dom import XML_NAMESPACE, XMLNS_NAMESPACE
from xml.dom.ext import SplitQName
from Ft.Lib import Uri

from pDomlette import Node, Document, Element, Attribute, DocumentFragment, \
     Text, ProcessingInstruction, Comment, NamedNodeMap, ReleaseNode


class Handler:
    def __init__(self, resolveEntity=None, processIncludes=1,
                 visitedHrefs=None, indexStringValues=1, force8Bit=0,
                 uriResolver=None):
        self._visitedHrefs = visitedHrefs or []
        self._resolveEntity = resolveEntity
        self._processIncludes = processIncludes
        self._indexStringValues = indexStringValues
        self._force8Bit = force8Bit
        self._uriResolver = uriResolver or Uri.BASIC_RESOLVER
        if self._force8Bit:
            try:
                import codecs
            except:
                raise Exception("Don't use force8Bit on Python versions that don't support unicode")
            enc = codecs.lookup('utf-8')[0]
            self._encoder = lambda x, e=enc: e(x)[0]
        return

    def _completeTextNode(self):
        #Note some parsers don't report ignorable white space properly
        if self._currText and len(self._nodeStack) and self._nodeStack[-1].nodeType != Node.DOCUMENT_NODE:
            if self._preserveStateStack[-1] or string.strip(self._currText):
                new_text = Text(self._ownerDoc)
                new_text.data = self._currText
                top_node = self._nodeStack[-1]
                top_node.appendChild(new_text)
                if self._indexStringValues:
                    top_node.stringValue = top_node.stringValue + self._currText
        self._currText = ''
        return

    def _handleStartElementNss(self, name, attribs):
        self._namespaces.append(self._namespaces[-1].copy())
        for curr_attrib_key in attribs.keys():
            (prefix, local) = SplitQName(curr_attrib_key)
            if not local:
                raise FtException(Error.INVALID_XMLNS, curr_attrib_key)
            if local == 'xmlns':
                self._namespaces[-1][prefix] = attribs[curr_attrib_key]
        (prefix, local) = SplitQName(name)

        if prefix and not self._namespaces[-1].has_key(prefix):
            raise FtException(Error.XMLNS_UNKNOWN_PREFIX, prefix)

        if self._namespaces[-1].has_key(prefix):
            namespace = self._namespaces[-1][prefix]
        else:
            namespace = ''
        attrs = {}
        qnames = {}
        for curr_attrib_key in attribs.keys():
            (a_prefix, a_local) = SplitQName(curr_attrib_key)
            if a_local == 'xmlns':
                ns = XMLNS_NAMESPACE
                a_local = a_prefix
            elif a_prefix and self._namespaces[-1].has_key(a_prefix):
                ns = self._namespaces[-1][a_prefix]
            else:
                ns = ''
            attrs[(ns, a_local)] = attribs[curr_attrib_key]
            qnames[(ns, a_local)] = curr_attrib_key

        nsattribs = xmlreader.AttributesNSImpl(attrs, qnames)
        return ((namespace, local), name, nsattribs)

    def _handleIncludes(self, name, qname, nsattribs):
        from Ft.Lib.XInclude import XINCLUDE_NAMESPACE, XIncludeException
        if self._includeDepth:
            self._includeDepth = self._includeDepth + 1
            return 0
        if not nsattribs.has_key((XINCLUDE_NAMESPACE, 'href')):
            return 0
        href = nsattribs[XINCLUDE_NAMESPACE, 'href']
        if href in self._visitedHrefs:
            raise XIncludeException(
                XIncludeException.CIRCULAR_INCLUDE_ERROR
                )
        parse = None
        if nsattribs.has_key((XINCLUDE_NAMESPACE, 'parse')):
            parse = nsattribs[(XINCLUDE_NAMESPACE, 'parse')]
        if not parse or parse == 'xml':
            context = qname
            curr_base = self.parser.GetBase()
            new_base = urllib.basejoin(curr_base, href)
            stream = self._uriResolver.resolve(href, curr_base)
            if self._entity_stack is None: self._entity_stack = []
            self._entity_stack.append(self.parser)

            self.parser= self.parser.ExternalEntityParserCreate(context)
            self.parser.SetBase(new_base)
            self._visitedHrefs.append(href)
            self.parser.ParseFile(stream)
            self._visitedHrefs.remove(href)

            self.parser= self._entity_stack.pop()
            stream.close()
        else:
            if os.path.exists(href):
                st = open(href)
            else:
                st = urllib.open(href)
            self._currText = self._currText + st.read()
            st.close()
        self._includeDepth = 1
        return 1

    def initState(self, ownerDoc=None, stripElements=None):
        self._preserveStateStack = [1]
        self._stripElements = stripElements or []
        if ownerDoc:
            self._ownerDoc = ownerDoc
            #Create a docfrag to hold all the generated nodes.
            self._rootNode = DocumentFragment(ownerDoc)
        else:
            self._rootNode = self._ownerDoc = Document()
        #Set up the stack which keeps track of the nesting of DOM nodes.
        #if self._indexStringValues: print "RN:", self._rootNode; self._rootNode.stringValue = ""
        if self._indexStringValues: self._rootNode.stringValue = ""
        self._nodeStack = [self._rootNode]
        self._namespaces = [{'xml': XML_NAMESPACE}]
        self._currText = ''
        self._includeDepth = 0
        return

    def processingInstruction(self, target, data):
        if self._force8Bit:
            target = self._encoder(target)
            data = self._encoder(data)
        if self._includeDepth: return
        self._completeTextNode()
        pi = ProcessingInstruction(self._ownerDoc)
        pi.target = target
        pi.data = data
        self._nodeStack[-1].appendChild(pi)

    def comment(self, data):
        if self._force8Bit:
            data = self._encoder(data)
        if self._includeDepth: return
        self._completeTextNode()
        comment = Comment(self._ownerDoc)
        comment.data = data
        self._nodeStack[-1].appendChild(comment)

    def startElement(self, name, attribs):
        if self._force8Bit:
            name = self._encoder(name)
            new_attribs = {}
            for k in attribs.keys():
                new_attribs[self._encoder(k)] = self._encoder(attribs[k])
            attribs = new_attribs
        (name, qname, nsattribs) = self._handleStartElementNss(name, attribs)
        if self._processIncludes:
            self._handleIncludes(name, qname, nsattribs)
            if self._includeDepth: return
        namespace = name[0]
        local = name[1]
        prefix = SplitQName(qname)[0]
        self._completeTextNode()
        new_element = Element(self._ownerDoc, namespace, local, prefix)
        for attr_qname in nsattribs.getQNames():
            (attr_ns, attr_local) = nsattribs.getNameByQName(attr_qname)
            attr_prefix = SplitQName(attr_qname)[0]
            attr = Attribute(self._ownerDoc, attr_ns, attr_local, attr_prefix)
            attr.value = nsattribs.getValueByQName(attr_qname)
            attr.ownerElement = new_element
            new_element.attributes[(attr_ns, attr_local)] = attr
        new_pstate = self._preserveStateStack[-1]
        for (uri, local, strip) in self._stripElements:
            if (uri, local) in [(new_element.namespaceURI,
                                 new_element.localName), ('', '*'),
                                (new_element.namespaceURI, '*')
                                ]:
                new_pstate = not strip
                break
        self._preserveStateStack.append(new_pstate)
        self._nodeStack.append(new_element)
        if self._indexStringValues: new_element.stringValue = ""
        return

    def endElement(self, name):
        if self._force8Bit:
            name = self._encoder(name)
        del self._namespaces[-1]
        if self._includeDepth:
            self._includeDepth = self._includeDepth - 1
            return
        self._completeTextNode()
        new_element = self._nodeStack[-1]
        del self._preserveStateStack[-1]
        del self._nodeStack[-1]
        self._nodeStack[-1].appendChild(new_element)
        return

    def characters(self, data):
        if self._force8Bit:
            data = self._encoder(data)
        if self._includeDepth: return
        self._currText = self._currText + data
        return

    _entity_stack= None
    def entityRef(self, context, base, sysid, pubid):
        if self._force8Bit:
            base = self._encoder(base)
            sysid = self._encoder(sysid)
            pubid = self._encoder(pubid)
        stream = self.resolveEntity(pubid, _combineSysid(base, sysid))

        if self._entity_stack is None: self._entity_stack = []
        self._entity_stack.append(self.parser)

        self.parser= self.parser.ExternalEntityParserCreate(context)
        self.parser.SetBase(_getBase(stream))
        self.parser.ParseFile(stream)

        self.parser= self._entity_stack.pop()
        stream.close()
        return 1

    def resolveEntity(self, pubid, sysid):
        if self._force8Bit:
            sysid = self._encoder(sysid)
            pubid = self._encoder(pubid)
        if self._resolveEntity is None: source= sysid
        else: source= self._resolveEntity(pubid, sysid)

        if type(source) in stringTypes:
            source= urllib.urlopen(source)

        return source

    def getRootNode(self):
        self._completeTextNode()
        return self._rootNode or self._ownerDoc


#FIXME: This is not a proper mix-in
class ReaderMixin:
    def fromStream(self, stream, refUri='', ownerDoc=None,
                   stripElements=None):
        self.initParser()
        self.handler.initState(ownerDoc, stripElements)
        p = self.parser
        p.SetBase(refUri)
        try:
            self.parser.ParseFile(stream)
        except Exception, e:
            #FIXME: This makes poor assumptions of the handler: encapsulate.
            if self.handler._rootNode: ReleaseNode(self.handler._rootNode)
            if self.handler._ownerDoc: ReleaseNode(self.handler._ownerDoc)
            if p.ErrorCode:
                raise FtException(Error.XML_PARSE_ERROR,
                                  p.ErrorLineNumber,
                                  p.ErrorColumnNumber,
                                  expat.ErrorString(p.ErrorCode))
            else:
                raise
        rootNode = self.handler.getRootNode()
        self.killParser()
        return rootNode
    

class PyExpatReader(ReaderMixin, ReaderBase.DomletteReader):
    HandlerClass = Handler
    def __init__(self, resolveEntity=None, processIncludes=1,
                 visitedHrefs=None, force8Bit=0):
        ReaderBase.DomletteReader.__init__(self, force8Bit)
        self._resolveEntity = resolveEntity
        self._processIncludes = processIncludes
        self._visitedHrefs = visitedHrefs
        return

    def releaseNode(self, node):
        ReleaseNode(node)

    def initParser(self):
        self.handler = self.HandlerClass(resolveEntity=self._resolveEntity,
                                         processIncludes=self._processIncludes,
                                         visitedHrefs=self._visitedHrefs,
                                         force8Bit=self._force8Bit)
        self.parser=expat.ParserCreate()
        self.parser.StartElementHandler = self.handler.startElement
        self.parser.EndElementHandler = self.handler.endElement
        self.parser.CharacterDataHandler = self.handler.characters
        self.parser.ProcessingInstructionHandler = self.handler.processingInstruction
        self.parser.CommentHandler = self.handler.comment
        self.parser.ExternalEntityRefHandler = self.handler.entityRef
        self.handler.parser = self.parser
        return

    def killParser(self):
        self.parser = None
        self.handler = None

class DomletteGenerator(Handler, XmlDomGenerator):
    def __init__(self, keepAllWs=0, processIncludes=1,
                 visitedHrefs=None, indexStringValues=1):
        Handler.__init__(self, processIncludes,
                         visitedHrefs, indexStringValues)
        XmlDomGenerator.__init__(self, keepAllWs)
        return

    def getRootNode(self):
        self._completeTextNode()
        return self._rootNode

    def startElementNS(self, name, qname, attribs):
        namespace = name[0]
        local = name[1]
        prefix = SplitQName(qname)[0]
        new_element = Element(self._ownerDoc, namespace, local, prefix)
        for attr_qname in attribs.getQNames():
            (attr_ns, attr_local) = attribs.getNameByQName(attr_qname)
            attr_prefix = SplitQName(attr_qname)[0]
            attr = Attribute(self._ownerDoc, attr_ns, attr_local, attr_prefix)
            attr.value = attribs.getValueByQName(attr_qname)
            attr.ownerElement = new_element
            new_element.attributes[(attr_ns, attr_local)] = attr
        self._nodeStack.append(new_element)
        return

    def endElementNS(self, name, qname):
        self._completeTextNode()
        new_element = self._nodeStack[-1]
        del self._preserveStateStack[-1]
        del self._nodeStack[-1]
        del self._namespaces[-1]
        self._nodeStack[-1].appendChild(new_element)
        return


#class SaxReader():
class SaxReader(ReaderMixin, ReaderBase.DomletteReader):
    def __init__(self, validate=0, keepAllWs=0, catName=None,
                 saxHandlerClass=DomletteGenerator, parser=None,
                 force8Bit=0):
        ReaderBase.DomletteReader.__init__(self, force8Bit)
        self._origParser = parser
        self._validate = validate
        self._keepAllWs = keepAllWs
        self._catName = catName
        self._saxHandlerClass = saxHandlerClass
        return

    def __getinitargs__(self):
        return (self.validate, self._keepAllWs, self._catName,
                self._saxHandlerClass, self._parser)

    def releaseNode(self, node):
        ReleaseNode(node)

    def initParser(self):
        self.parser = self._origParser or (self._validate and sax2exts.XMLValParserFactory.make_parser()) or sax2exts.XMLParserFactory.make_parser()
        if self._catName:
            #set up the catalog, if there is one
            from xml.parsers.xmlproc import catalog
            cat_handler = catalog.SAX_catalog(self._catName, catalog.CatParserFactory())
            self.parser.setEntityResolver(self._cat_handler)
        self.handler = self._saxHandlerClass(self._keepAllWs)
        if self._force8Bit:
            self.handler = Utf8OnlyHandler(self.handler)
        self.parser.setContentHandler(self.handler)
        self.parser.setDTDHandler(self.handler)
        self.parser.setErrorHandler(self.handler)
        self.parser.setProperty(handler.property_lexical_handler, self.handler)
        try:
            self.parser.setProperty(handler.property_declaration_handler,
                                    self.handler)
        except (SystemExit, KeyboardInterrupt):
            raise
        except:
            pass
        return

    def killParser(self):
        self.parser = None
        self.handler = None


def _combineSysid(base,sysid):
    return urllib.basejoin(base, sysid)


def _getBase(stream):
    #FIXME: This is an odd request.  What does it mean to get the "base"
    #Of an arbitrary URI?
    name = getattr(stream, 'url', None) or getattr(stream, 'name', '')
    return name


