

from Ft.Ods import Database
from Ft.Ods.Collections import CollectionBase
from Ft.Ods.StorageManager.Adapters import Constants

Actions = Constants.CollectionChanges

import types

g_toOdmgType = {types.StringType:Constants.Types.STRING,
                types.InstanceType:Constants.Types.OBJECT,
                types.IntType:Constants.Types.SIGNED_SHORT,
                types.LongType:Constants.Types.SIGNED_LONG_LONG,
                types.FloatType:Constants.Types.DOUBLE,
                }

g_odmgTypeToString = {Constants.Types.STRING : 'string',
                      Constants.Types.OBJECT : 'object',
                      Constants.Types.SIGNED_SHORT : 'short',
                      Constants.Types.SIGNED_LONG_LONG : 'long long',
                      Constants.Types.DOUBLE : 'double',
                      }

g_modifyThreshold = 0.50  # percentage of changes allowed to original data

class CollectionOfValues(CollectionBase.CollectionBase):

    def __init__(self, cid=None,subType=None, values=None):

        self.__dict__['_dbChanges'] = []

        if cid is not None:
            # Recreate an existing object
            # map values to a list of tuples:
            #  (db_idx, data)

            values = map(None, range(len(values)), values)
            self.__dict__['_dbSize'] = len(values)
        else:
            self.__dict__['_dbSize'] = 0

        CollectionBase.CollectionBase.__init__(self,cid,subType,values)

        return

    def __getattr__(self,name):
        #Support for the value attribute
        if name == 'value':
            return map(lambda (idx,object):object,self._values)
        raise AttributeError(name)

    #4ODS interfaces

    def _4ods_setTransaction(self, tx, unregister=0):
        CollectionBase._4ods_setTransaction(self, tx, unregister)
        if tx:
            self.__dict__['_dbChanges'] = []
        return


    def _4ods_getChanges(self):
        changes = self._dbChanges
        self.__dict__['_dbSize'] = len(self._values)
        self.__dict__['_dbChanges'] = []
        return changes
        #NOTE Optimize by checking size to num of operations
        #if not self._initialized:
        #    # Return the whole thing, not in the database yet
        #    changes = (None, contents)
        #elif len(self._dbChanges) < (self._dbSize * g_modifyThreshold):
        #    # Number of changes less than the limit, return just the changes
        #    changes = (self._dbChanges, None)
        #else:
        #    # Threshold reached, return all values
        #    changes = (None, contents)
        #self.__dict__['_initialized'] = 1
        #return changes

    def _4ods_isModified(self):
        return len(self._dbChanges) > 0

    def _4ods_getSubType(self):
        return self._subType



    #Override some of the pythonic interfaces
    def append(self, element):
        if self._subType is None:
            # Blindly set the type of collection to this
            self.__dict__['_subType'] = g_toOdmgType[type(element)]
        elif self._subType != g_toOdmgType[type(element)]:
            raise TypeError('Expected %s, got %s' % (
                g_odmgTypeToString[self._subType],
                g_odmgTypeToString[g_toOdmgType[type(element)]],
                ))
        self._values.append((None, element))
        if len(self._dbChanges) and self._dbChanges[-1][0] == Actions.APPEND:
            self._dbChanges[-1][1].append((element,len(self._values)-1))
        else:
            self._dbChanges.append((Actions.APPEND, [(element,len(self._values)-1)]))
        return 

    def insert(self, i, element):
        if self._subType is None:
            # Blindly set the type of collection to this
            self.__dict__['_subType'] = g_toOdmgType[type(element)]
        elif self._subType != g_toOdmgType[type(element)]:
            raise TypeError('Expected %s, got %s' % (
                g_odmgTypeToString[self._subType],
                g_odmgTypeToString[g_toOdmgType[type(element)]],
                ))
        self._values.insert(i, (None, element))
        if len(self._dbChanges) and self._dbChanges[-1][0] == Actions.INSERT:
            self._dbChanges[-1][1].append((element, i))
        else:
            self._dbChanges.append((Actions.INSERT, [(element, i)]))
        return 

    def remove(self, item):
        ctr = 0
        for (idx,element) in self._values:
            if element == item:
                del self[ctr]
                #NOTE delitem adjusts the dbChanges
                break
            ctr = ctr + 1
        else:
            raise ValueError('%s not in collection' % str(element))

    def count(self, item):
        return len(filter(lambda (idx,object),i=item:object==i,self._values))

    def index(self, item):
        contents = map(lambda (idx, object): object, self._values)
        return contents.index(item)

    def __getitem__(self, i):
        # Return the data portion of the tuple
        return self._values[i][1]

    def __setitem__(self, i, item):
        del self[i]
        self.insert(i,item)

    def __delitem__(self, i):
        value = self._values.pop(i)
        if len(self._dbChanges) and self._dbChanges[-1][0] == Actions.REMOVE:
            self._dbChanges[-1][1].append(i)
        else:
            self._dbChanges.append((Actions.REMOVE, [i]))


