############################################################################
#
# File Name: 		DictionaryOfPersistentObjects.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/DictionaryOfPersistentObjects.py.html
#
"""
Base class for all dictionaries of persistent objects
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods import Database

from Ft.Ods import Collections


from Ft.Ods.Collections import Iterator
from Ft.Ods.Collections import BidirectionalIterator

class DictionaryOfPersistentObjects:

    def __init__(self,_type,keyType,valueClass):

        #{key:(id,inst)}
        self.__objs = {}
        self.__db = Database.Database()
        self.__modified = 0
        self.__keyType = keyType
        self.__valueClass = valueClass
        self.__type = _type
        if type(self.__type) != type(1):
            raise "Dumb Ass"

        self.__initargs = (keyType,valueClass)

    def __getinitargs__(self):
        return self.__initargs


    def bind(self,key,value):
        if self.__objs.has_key(key):
            raise Collections.DuplicateKey(key)

        if not type(key) == self.__keyType:
            raise "Invalid Key Type"

        if type(value) == type(1):
            self.__objs[key] = (value,None)
        elif isinstance(value,self.__valueClass):
            self.__objs[key] = (value._4ods_getId(),value)
        else:
            raise "Wrong Type"
        self.__modified = 1

    def unbind(self,key):
        if not self.__objs.has_key(key):
            raise Collections.KeyNotFound(key)
        del self.__objs[key]
        self.__modified = 1

    def lookup(self,key):
        if not self.__objs.has_key(key):
            raise Collections.KeyNotFound(key)
        if not self.__objs[key][1]:
            id = self.__objs[key][0]
            self.__objs[key] = (id,self.__db._4ods_getPersistentObjectById(self.__type,id))
        return self.__objs[key][1]

    def contains_key(self,key):
        return self.__objs.has_key(key)


    #Interfaces from collection

    def is_ordered(self):
        return 1
    def allows_duplicates(self):
        return 0
    def cardinality(self):
        return len(self.__objs.keys())
    def contains_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"
        if not self.__objs.has_key(ass.key):
            return 0

        entry = self.lookup(ass.key)
        return ass.value == entry

    def insert_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"
        if self.__objs.has_key(ass.key):
            self.unbind(ass.key)
        self.bind(ass.key,ass.value)


    def remove_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"


        if not self.__objs.has_key(ass.key):
            return

        if self.lookup(ass.key) == ass.value:

            del self.__objs[ass.key]


    def copy(self,deep=1):
        initArgs = ()
        if hasattr(self,'__getinitargs__'):
            initArgs = self.__getinitargs__()
           
        other = apply(self.__class__,initArgs)

        if deep:
            other._4ods_initialize(self._4ods_getContents())
        return other        


    def create_iterator(self, stable):
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")




    def _4ods_isModified(self):
        return self.__modified

    def _4ods_initialize(self,data):
        for k,v in map(lambda x,y: (x,y),data[0],data[1]):
            if type(v) == type(1):
                self.__objs[k] = (v,None)
            else:
                self.__objs[k] = (v._4ods_getId(),v)

    def _4ods_clearContents(self):
        self.__objs = {}
        self.__modified = 0

    def _4ods_setModified(self):
        self.__modified = 1
    

    #Pythonic interface

    def __cmp__(self, dict):
        if isinstance(dict,DictionaryOfPersistentObjects):
            return self.keys() == dict.keys() and self.values() == dict.values()
        elif isinstance(dict, UserDict):
            return self.keys() == dict.keys() and self.values() == dict.values()
        else:
            return cmp(self.keys(), dict)
    def __len__(self): return len(self.keys())
    def __getitem__(self, key): return self.lookup(key)
    def __setitem__(self, key, item): self.bind(key,item)
    def __delitem__(self, key): self.unbind(key)
    def clear(self):
        for key in self.keys():
            self.unbind(key)
    def keys(self): return self.__objs.keys()
    def items(self):
        rt = []
        for key in self.keys():
            rt.append(key,self.lookup(key))
        return rt
    def values(self):
        rt = []
        for key in self.keys():
            rt.append(self.lookup(key))
        return rt

    def has_key(self, key): return self.__objs.has_key(key)

    def update(self,dict):
        for k, v in dict.items():
            self[k] = v
    def get(self, key, failobj=None):
        if self.has_key(key):
            return self[key]
        return failobj
    def _4ods_retrieveElementByIndex(self,index):
        return self.values()[index]
