############################################################################
#
# File Name: 		LiteralDictionaryOfValues.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/LiteralDictionaryOfValues.py.html
#
"""

WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import UserDict

from Ft.Ods import Collections
from Ft.Ods import Literal

from Ft.Ods.Collections import Iterator
from Ft.Ods.Collections import BidirectionalIterator

from Ft.Ods.StorageManager.Adapters import Constants

class LiteralDictionaryOfValues(UserDict.UserDict,Literal.Literal):
    
    def __init__(self,keyType,valueType):
        UserDict.UserDict.__init__(self)
        Literal.Literal.__init__(self)
        self.__keyType = keyType
        self.__valueType = valueType

        self.__initargs = (keyType,valueType)

    def __getinitargs__(self):
        return self.__initargs


    def bind(self,key,value):
        if self.has_key(key):
            raise Collections.DuplicateKey(key)

        if not type(key) == self.__keyType:
            raise "Invalid Key Type"
        self[key] = value
        self.__modified = 1

    def unbind(self,key):
        if not self.has_key(key):
            raise Collections.KeyNotFound(key)
        del self[key]
        self.__modified = 1

    def lookup(self,key):
        if not self.has_key(key):
            raise Collections.KeyNotFound(key)
        return self[key]

    def contains_key(self,key):
        return self.has_key(key)


    #Interfaces from collection

    def is_ordered(self):
        return 1
    def allows_duplicates(self):
        return 0
    def cardinality(self):
        return len(self.keys())
    def contains_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"
        if not self.has_key(ass.key):
            return 0

        return self[ass.key] == ass.value

    def insert_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"
        self[ass.key] = ass.value

    def remove_element(self,ass):
        if not isinstance(ass,Collections.Association):
            raise "Must be an association"

        if not self.has_key(ass.key):
            return
        if self[ass.key] == ass.value:
            del self[ass.key]

    def copy(self,deep=1):
        initArgs = ()
        if hasattr(self,'__getinitargs__'):
            initArgs = self.__getinitargs__()
           
        other = apply(self.__class__,initArgs)

        if deep:
            other._4ods_initialize(self._4ods_getContents())
        return other        


    def create_iterator(self, stable):
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")



    def _4ods_isModified(self):
        return self.__modified

    def _4ods_initialize(self,data):
        for k,v in map(lambda x,y: (x,y),data[0],data[1]):
            self[k] = v

    def _4ods_clearContents(self):
        self.clear()
        self.__modified = 0

    def _4ods_setModified(self):
        self.__modified = 1


    def _4ods_retrieveElementByIndex(self,index):
        return self.values()[index]



    #Dictionary interface
    def _4ods_getLiteralType(self):
        return Constants.LiteralType.DICTIONARY

