########################################################################
#
# File Name: 	       Collection.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Collection.py.html
#
"""
Implements the Collection meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Ods.MetaData import Type
import string

from Ft.Ods.MetaData import MetaKind
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.MetaData import CollectionKind
from Ft.Ods.PersistentObject import TupleDefinitions
td = TupleDefinitions


g_typeMap = {
    Constants.Types.BOOLEAN: {'ck_bag':Constants.Types.BOOLEAN_BAG,
                              'ck_set':Constants.Types.BOOLEAN_SET,
                              'ck_list':Constants.Types.BOOLEAN_LIST
                              },
    Constants.Types.SIGNED_SHORT:   {'ck_bag':Constants.Types.SIGNED_SHORT_BAG,
                                     'ck_set':Constants.Types.SIGNED_SHORT_SET,
                                     'ck_list':Constants.Types.SIGNED_SHORT_LIST
                              },
    Constants.Types.STRING:   {'ck_bag':Constants.Types.STRING_BAG,
                              'ck_set':Constants.Types.STRING_SET,
                              'ck_list':Constants.Types.STRING_LIST
                              },
    Constants.Types.UNSIGNED_SHORT:   {'ck_bag':Constants.Types.UNSIGNED_SHORT_BAG,
                                     'ck_set':Constants.Types.UNSIGNED_SHORT_SET,
                                     'ck_list':Constants.Types.UNSIGNED_SHORT_LIST
                              },
    Constants.Types.SIGNED_LONG:   {'ck_bag':Constants.Types.SIGNED_LONG_BAG,
                                     'ck_set':Constants.Types.SIGNED_LONG_SET,
                                     'ck_list':Constants.Types.SIGNED_LONG_LIST
                              },
    
    Constants.Types.UNSIGNED_LONG:   {'ck_bag':Constants.Types.UNSIGNED_LONG_BAG,
                                      'ck_set':Constants.Types.UNSIGNED_LONG_SET,
                                      'ck_list':Constants.Types.UNSIGNED_LONG_LIST
                                      },
    
    Constants.Types.SIGNED_LONG_LONG:   {'ck_bag':Constants.Types.SIGNED_LONG_LONG_BAG,
                                         'ck_set':Constants.Types.SIGNED_LONG_LONG_SET,
                                         'ck_list':Constants.Types.SIGNED_LONG_LONG_LIST
                                         },
    Constants.Types.FLOAT:   {'ck_bag':Constants.Types.FLOAT_BAG,
                              'ck_set':Constants.Types.FLOAT_SET,
                              'ck_list':Constants.Types.FLOAT_LIST
                              },
    Constants.Types.DOUBLE:   {'ck_bag':Constants.Types.DOUBLE_BAG,
                               'ck_set':Constants.Types.DOUBLE_SET,
                               'ck_list':Constants.Types.DOUBLE_LIST
                               },
    
    Constants.Types.OBJECT: {'ck_list':Constants.Types.OBJECT_LIST,
                             'ck_bag':Constants.Types.OBJECT_BAG,
                             'ck_set':Constants.Types.OBJECT_SET
                             }
    }

class Collection(Type.Type):

    def __init__(self,data=None,meta_kind = MetaKind.MetaKind.mk_collection,collection_kind=None):
        Type.Type.__init__(self,data,meta_kind)
        if data is None:
            if collection_kind is None:
                raise "No Collection Kind"
            self.__dict__['collection_kind'] = collection_kind

    def form_subtype(self,target,inverse = 1):
        self._4ods_formRelationship('subtype',Type.Type,'collections','add',target,inverse)

    def drop_subtype(self, target, inverse=1):
        self._4ods_dropRelationship('subtype','collections','remove',target,inverse)

    def form_max_size(self,target,inverse = 1):
        from Ft.Ods.MetaData import Operand
        self._4ods_formRelationship('max_size',Operand.Operand,'size_of','form',target,inverse)

    def drop_max_size(self, target, inverse=1):
        self._4ods_dropRelationship('max_size','size_of','drop',target,inverse)

    def is_ordered(self):
        return self.collection_kind == CollectionKind.CollectionKind.ck_list

    def _4ods_getOdl(self):
        if self.collection_kind == CollectionKind.CollectionKind.ck_list:
            return "List"
        if self.collection_kind == CollectionKind.CollectionKind.ck_set:
            return "Set"
        if self.collection_kind == CollectionKind.CollectionKind.ck_bag:
            return "Bag"

    def _4ods_getOdmgType(self):
        global g_typeMap
        return g_typeMap[self.subtype._4ods_getOdmgType()][self.collection_kind._n]

    _tupleNames =  ('collection_kind','max_size','subtype')


    _tupleDefinitions = {'collection_kind': {td.TYPE:Constants.Types.ENUMERATION,
                                             td.READONLY:1,
                                             td.ENUM_CLASS:CollectionKind.CollectionKind,
                                             },
                         'max_size': {td.TYPE:Constants.Types.OBJECT,
                                      td.RELATIONSHIP:1,
                                      td.READONLY:1,
                                      },
                         'subtype': {td.TYPE:Constants.Types.OBJECT,
                                     td.RELATIONSHIP:1,
                                     td.READONLY:1,
                                     },
                         
                         }
    localExtents = ()

