########################################################################
#
# File Name: 	        PrimitiveType.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/PrimitiveType.py.html
#
"""
Implements the Type meta-data interface.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""


from Ft.Ods.MetaData import Type
from Ft.Ods.MetaData import MetaKind
from Ft.Ods.MetaData import PrimitiveKind
from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.PersistentObject import TupleDefinitions
td = TupleDefinitions


g_nameToOdlMap = {'pk_boolean' : 'boolean',
                  'pk_char' : 'char',
                  'pk_string' : 'string',
                  'pk_short' : 'short',
                  'pk_ushort' : 'unsigned short',
                  'pk_date' : 'date',
                  'pk_time' : 'time',
                  'pk_timestamp' : 'timestamp',
                  'pk_long' : 'long',
                  'pk_ulong' : 'unsigned long',
                  'pk_long_long' : 'long long',
                  'pk_float' : 'float',
                  'pk_double' : 'double',
                  'pk_octet' : 'octet',
                  'pk_interval' : 'interval',
                  'pk_void' : 'void',
                  'pk_blob' : 'blob'
                  }
g_nameToOdlType = {
    0:Constants.Types.BOOLEAN,
    2 : Constants.Types.STRING,
    3:Constants.Types.SIGNED_SHORT,
    4:Constants.Types.UNSIGNED_SHORT,
    8:Constants.Types.SIGNED_LONG,
    9:Constants.Types.UNSIGNED_LONG,
    10:Constants.Types.SIGNED_LONG_LONG,
    11: Constants.Types.FLOAT, 
    12:Constants.Types.DOUBLE,
    16:Constants.Types.BLOB
    }

class PrimitiveType(Type.Type):

    def __init__(self,data=None,pk=None):
        Type.Type.__init__(self,data,MetaKind.MetaKind.mk_primitive_type)
        if data is None:
            if pk is None:
                raise "No PK"
            self.__dict__['primitive_kind'] = pk


    def absolute_name(self):
        return g_nameToOdlMap[self.name]
        

    def _4ods_getOdmgType(self):
        return g_nameToOdlType[self.primitive_kind._v]

    
    def _4ods_getOdl(self):
        return g_nameToOdlMap[self.name]


    _tupleNames = ('primitive_kind',)
    _tupleDefinitions = {'primitive_kind':{td.TYPE:Constants.Types.ENUMERATION,
                                           td.READONLY:1,
                                           td.ENUM_CLASS:PrimitiveKind.PrimitiveKind,
                                           },
                         }

    _localExtents = ()


