#!/usr/bin/python
########################################################################
#
# File Name:            FILE
#
# Documentation:        http://docs.4suite.com/4ODS/Parsers/GenerateClass.py.html
#

#Desc.
#WWW: http://4suite.com/4ODS        e-mail: support@4suite.com

#Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
#See  http://4suite.com/COPYRIGHT  for license and copyright information

import string, os, cStringIO

from Ft.Ods.Parsers.Odl import GenerateUtils
from Ft.Ods.Parsers.Odl import GenerateType
from Ft.Ods.Parsers.Odl import Templates

from Ft.Ods.MetaData.MetaKind import MetaKind
from Ft.Ods.MetaData.PrimitiveKind import PrimitiveKind

def GenerateStubs(klass, params):
    # Shift directories
    classDir = os.path.join(params.curDir, klass.name)
    if not os.path.exists(classDir):
        os.makedirs(classDir)

    params.curDir = classDir

    # Write our file
    outFileName = os.path.join(params.curDir, '__init__.py')
    outFile = open(outFileName,'w')
    params.outFile = outFile
    outFile.write('# DO NOT EDIT THIS FILE!\n')
    outFile.write('#\n')
    outFile.write('# Python class %s generated by OdlParse\n' % klass.name)
    outFile.write("# Please subclass rather than editing.\n")
    # Print the imports


    bases = Templates.GetBases(klass.name, klass.inherits, params)
    if klass.extender:
        bases.extend(Templates.GetBases(klass.name, [klass.extender], params))

    if not bases:
        bases.append('Object.Object')
        outFile.write('from Ft.Ods import Object\n')

    outFile.write('from Ft.Ods import Database\n\n')

    for define in klass.defines:
        if define.meta_kind == MetaKind.mk_attribute:
            if define.type.meta_kind == MetaKind.mk_enumeration:
                GenerateType.WriteImports(define.type, params)
        
    Templates.GlobalFactory(outFile.write, klass.name)

    # Write the class definition and the constructor
    Templates.ClassHeader(outFile.write, klass.name, bases)

    # FIXME: no support for property_lists

    # Write the init for attributes
    attrs = filter(lambda define:
                   define.meta_kind == MetaKind.mk_attribute,
                   klass.defines)
    Templates.AttributeInit(attrs, params)

    # Write the init for structures
    structs = filter(lambda define:
                     define.meta_kind == MetaKind.mk_structure,
                     klass.defines)
    for struct in structs:
        raise NotImplementedError('FIXME')

    # Write the functions for relationships
    relationships = filter(lambda define:
                           define.meta_kind == MetaKind.mk_relationship,
                           klass.defines)
    for relationship in relationships:
        Templates.RelationshipInit(relationship, params)
    
    # Write the tuple info
    Templates.TupleDefinitions(klass.defines, bases, params)

    # Write the extents tuple
    Templates.InheritedTuples(outFile.write, '_extents', klass.extents, bases)

    outFile.write('    _typeId = %d\n' % klass._4ods_getId())

    # Write any imports we need for any object attributes and object relationsips
    for define in klass.defines:
        if define.meta_kind == MetaKind.mk_attribute:
            #if define.type.meta_kind == MetaKind.mk_enumeration:
            #    GenerateType.WriteImports(define.type, params)
            pass
        elif define.meta_kind != MetaKind.mk_relationship:
            GenerateType.WriteImports(define.type, params)

    outFile.write('\n')
    outFile.close()

    # Last but not least, see if any of defines need to do thier own generation
    for define in klass.defines:
        if define.meta_kind in [MetaKind.mk_constant,
                                ]:
            GenerateUtils.ExecuteDefinition(define, 'GenerateStubs', params)
