########################################################################
#
# File Name:            GenerateStructure.py
#
# Documentation:        http://docs.4suite.com/4ODS/Parsers/GenerateStructure.py.html
#
"""
Structured Literals
WWW: http://4suite.com/4ODS        e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string


from Ft.Ods.Parsers import GenerateMember
from Ft.Ods.Parsers import GenerateType

from Ft.Ods.MetaData import ScopedNameType

import os

def GenerateStubs(struct,params):
    #Shift directories

    params = params.copy()

    structDir = os.path.join(params.curDir, struct.name)
    if not os.path.exists(structDir):
        os.makedirs(structDir)

    params.curDir = structDir
    if params.curPackage:
        params.curPackage = params.curPackage + '.'
    params.curPackage = params.curPackage + struct.name

    #Write our file
    structName = struct.name
    outFileName = os.path.join(params.curDir, '__init__.py')
    outFile = open(outFileName,'w')
    params.outFile = outFile
    #print the imports
    outFile.write("#Generated file.  Please don't edit.\n")

    members = struct.fields

    for m in members:
        GenerateMember.WriteImports(m,params)

    outFile.write('from Ft.Ods import Literal\n\n')

        
    #Write a global factory
    outFile.write('class %sFactory:\n'%structName)
    outFile.write('    def new(self, db=None):\n')
    outFile.write('        return %s_stub(db)\n'%structName)
    outFile.write('g_%sFactory = %sFactory()\n'%(structName, structName))
    outFile.write('def new(db):\n')
    outFile.write('    g_%sFactory.new(db)\n\n' % structName)

    #Write the struct definition and the constructor
    outFile.write('class %s_stub(Literal.Literal):\n' % structName)

    outFile.write('    attributes = [')

    for m in members:
        outFile.write("'" + m.name + "'")
        if m != members[-1]:
            outFile.write(',')
    outFile.write(']\n')

    outFile.write("    def __init__(self, db=None, lid=0):\n")


    #Write the initializer for all of the attributes
    for m in members:
        outFile.write('        self.__dict__["%s"] = '%m.name)
        GenerateMember.WriteInit(m,params)
        outFile.write('\n')

    outFile.write('        Literal.Literal.__init__(self,db,lid)\n')


    #Write the setattr overide
    params.outFile.write("\n    def __setattr__(self, name, value):\n")
    params.outFile.write("        if name in %s_stub.attributes:\n"%struct.name)
    params.outFile.write("            if isinstance(self.__dict__[name],Literal.Literal):\n")
    params.outFile.write("                value._ftods_copyLiteral(self.__dict__[name])\n")
    params.outFile.write("            else:\n")
    params.outFile.write("                self._ftods_setModified(name)\n")
    params.outFile.write("                self.__dict__[name] = value\n")
    params.outFile.write("            return 1\n")
    params.outFile.write("        return Literal.Literal.__setattr__(self,name,value)\n")


    #Write copy method for literal
    params.outFile.write("    def _ftods_copyLiteral(self,target):\n")
    params.outFile.write("        for name in %s_stub.attributes:\n" % struct.name)
    params.outFile.write("            setattr(target,name,getattr(self,name))\n")


    structPath = string.split(struct._ftods_getScopedName(),'::')
    #Write new instance method
    params.outFile.write("\n    def _ftods_newInstance(self):\n")
    params.outFile.write("        lid = self._dba.newStructure(%s, self._ftods_getLid())\n" % str(structPath))
    params.outFile.write("        self._ftods_setLid(lid)\n")

    #Write commit logic
    params.outFile.write('    def _ftods_commit(self, modifiedAttrs = []):\n')
    params.outFile.write("        if self._committed: return\n")

    params.outFile.write("        Literal.Literal._ftods_commit(self,modifiedAttrs)\n")


    params.outFile.write("\n    def _ftods_commitAttributes(self):\n")
    params.outFile.write("        pass;\n")
    for m in members:
        if isinstance(m.type,ScopedNameType.ScopedNameType):
            params.outFile.write("        if self.%s and not self.%s._ftods_getLid():\n" % (m.name,m.name))
            params.outFile.write("            self.%s._ftods_commit()\n" % m.name)



    #Write read logic
    params.outFile.write("\n    def _ftods_read(self):\n")
    params.outFile.write("        data = self._dba.readStructData(self, %s)\n" % str(structPath))
    params.outFile.write("        self._ftods_initFromRead(data)\n")
    params.outFile.write('\n\n')
    params.outFile.write('    def _ftods_initFromRead(self,data):\n')

    for m in members:
        params.outFile.write('        curVal = data["%s"]\n'%m.name)
        params.outFile.write('        self.__dict__["%s"] = ' % m.name)
        GenerateMember.WriteRead(m,params)
        params.outFile.write("\n")
    
    params.outFile.write("        Literal.Literal._ftods_initFromRead(self,data)\n")
        
    #Write write logic
    params.outFile.write("    def _ftods_write(self,modifiedAttrs = []):\n")
    params.outFile.write("        data = []\n")
    params.outFile.write("        self._ftods_setWriteData(data)\n")

    params.outFile.write("        self._dba.writeStructData(self, data, %s)\n"%str(structPath))


    params.outFile.write("    def _ftods_setWriteData(self,data):\n")

    for m in members:
        params.outFile.write("        curVal = self.%s\n" % m.name)
        params.outFile.write("        curVal = ")
        GenerateMember.WriteWrite(m,params)
        params.outFile.write("\n")
        params.outFile.write("\n        data.append((%s,'%s', curVal))\n" % (str(structPath),m.name))

    #Write the rest of the stuff
    params.outFile.write("    def _ftods_delete(self,recurse=0):\n")
    params.outFile.write("        if self._deleted:\n")
    params.outFile.write("            return\n")
    params.outFile.write("        self._dba.deleteStruct(self, %s)\n"%str(structPath))
    params.outFile.write("        Literal.Literal._ftods_delete(self)\n\n")
    params.outFile.write("        if recurse:\n")
    params.outFile.write("            pass\n")
    for m in members:
        if isinstance(m.type,ScopedNameType.ScopedNameType):
            params.outFile.write("        if self.%s:\n" % m.name)
            params.outFile.write("            self.%s._ftods_delete()\n" % m.name)

    params.outFile.write("    def _ftods_getStructPath(self):\n")
    params.outFile.write("        return %s\n" % str(structPath))
    
    outFile.close()




#Functions to init the DB
def SetMetaId(struct,db):
    for m in struct.fields:
        GenerateMember.SetMetaId(m,db)


def InitDb(struct, db):
    #Now create the struct
    db.createStructure(struct)
    for m in struct.fields:
        GenerateMember.InitDb(m,db)

