#!/usr/bin/env python
########################################################################
#
# File Name: 	        OdlParse.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/OdlParse.py.html
#
"""
Main program to generate pstubs and initialize the database according to ODL input.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought LLC, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import string, os, sys

from Ft.Ods.StorageManager import Adapters
from Ft.Ods.StorageManager.Adapters import Dbm
from Ft.Ods.Parsers.Odl import OdlParserImp
from Ft.Ods.Parsers.Odl import OdlParserBase
from Ft.Ods import Database
TEMP_DBNAME="ods:nobody_else_can_use"
class FileGenParams:
    def __init__(self):
        self.curPackage = ''
        self.basePackage = ''
        self.curDir = ''
        self.baseDir = ''
        self.outFile = None
        self.repository = None
        self.types = {}
        self.indent = ''

    def copy(self):
        nfgp = FileGenParams()
        for k in self.__dict__.keys():
            nfgp.__dict__[k] = self.__dict__[k]
        return nfgp


def ReadOdl(cpp, odl_string, args):
    odlFileName = "<string>"
    try:
        if odl_string is None:
            odlFileName = args[1]
            if odlFileName == '-':
                odlFileName = "STDIN"
                odl_string = sys.stdin.read()
            else:
                odl_string = open(args[1], 'r').read()
    except IOError, e:
        print e
        raise 'Unable to open file: '+args[1]
    if cpp:
        rt = None
        cccp = None
        import tempfile
        outName = tempfile.mktemp()
        errName = tempfile.mktemp()
        try:
            #Run the code throug the CPP
            cccp = os.popen("gcc -E -P -x c++ - 1> %s 2> %s" % (outName,errName),'w')
            cccp.write(odl_string)
            rt = cccp.close()
            cccp = None
        finally:
            if cccp:
                rt = cccp.close()

        errFile = open(errName,'r')
        errLines = errFile.readlines()
        errFile.close()
        if len(errLines):
            #Houston, you know the rest
            print "*"*40
            print "Preprocessor Error"
            import re
            thisFile = re.compile("(^|\s):(?P<line>\d*)")
            otherFile = re.compile("(?P<file>.*?):(?P<line>\d*)")

            for line in errLines:
                thisFileRt = thisFile.search(line)
                otherFileRt = otherFile.search(line)
                if thisFileRt:
                    print "%s in %s line %s: %s" % (line[:thisFileRt.start()],odlFileName,thisFileRt.group('line'),line[thisFileRt.end('line')+1:-1])
                elif otherFileRt:
                    print "%s in %s line %s: %s" % (line[:otherFileRt.start('file')],otherFileRt.group('file'),otherFileRt.group('line'),line[otherFileRt.end('line')+1:-1])

            raise "Preprocessor Exception"

        outFile = open(outName,'r')
        odl_string = outFile.read()
        outFile.close()

        os.unlink(errName)
        os.unlink(outName)
    return odl_string


def parse(do_db,cpp,db_name,gen_pstubs,package,odl_string=None,args=[],baseDir='.'):

    
    db = None
    db = Database.Database()

    odl_string = ReadOdl(cpp,odl_string,args)

    if odl_string is None:
        raise "Must spcify ODL somehoe"



    if not do_db:
        #Read the ODL into a temporary DB
        #Force it to use the Dbm adapter
        Adapters.g_driverName = 'Dbm'
        mang = Adapters.GetManager()
        adapter = Adapters.GetAdapter()
        if mang.exists(TEMP_DBNAME):
            mang.destroy(TEMP_DBNAME)
        mang.new(TEMP_DBNAME,adapter)

        db_name = TEMP_DBNAME

    db = Database.Database()
    db.open(db_name)
    db.new().begin()
    try:
        parser = OdlParserImp.OdlParserImp()
        parser.parse(odl_string)
    except OdlParserBase.InternalException, e:
        OdlParserBase.PrintInternalException(e)
        sys.exit()
    except OdlParserBase.SyntaxException, e:
        OdlParserBase.PrintSyntaxException(e)
        sys.exit()

    tx = db.current()
    tx.commit()
    tx = db.new()
    tx.begin()

    #Validate the tree
    fp = FileGenParams()
    fp.baseDir = baseDir
    fp.curPackage = package
    fp.basePackage = package


    #Generate files if needed
    from Ft.Ods.Parsers.Odl import GenerateRepository
    if gen_pstubs:
        GenerateRepository.GenerateStubs(fp,db.schema().resolve('ODLMetaObjects'))
    tx = db.current()
    tx.abort()
    db.close()

    if not do_db:
        mang.destroy(TEMP_DBNAME)

def Run(argv):
    import sys, os, getopt
    import Ft
    from Ft.Ods.Parsers.Odl import PostProcessor

    usage_string = """
4ODS version %s

Usage: %s [options] <database name> <odl source>
Options:
  -p, --generate       Generate python stubs
    --package <name>   Put stub files in package name
    --directory <dir>  Directory to place pstubs (default: current)

  -i, --init           Initialize the specified database
    -f, --force        Overwrite any data in the database

  -u, --update         Update an existing database
  -c, --preprocess     Run the ODL through a preprocessor before compiling

  -h, --help           Display this information

If you use '-' as the name of the source, standard input will be used.
""" % (Ft.__version__, os.path.basename(argv[0]))

    short_opts = 'pifuch'
    long_opts = ['generate',
                 'package=',
                 'directory=',
                 'init',
                 'force',
                 'update',
                 'preprocess',
                 'help']
    
    if len(argv) <= 1:
        print usage_string
        sys.exit(1)

    optList, args = getopt.getopt(argv[1:], short_opts, long_opts)

    if not args:
        print "Database name required."
        print usage_string
        sys.exit(1)

    package = ''
    directory = '.'
    gen_pstubs = 0
    init_db = 0
    update_db = 0
    db_name = ''
    force = 0
    cpp = 0
    for (opt, value) in optList:
        if opt in ['-p', '--generate']:
            gen_pstubs = 1
        elif opt == '--package':
            package = value
        elif opt == '--directory':
            directory = value
        elif opt in ['-c', '--preprocess']:
            cpp = 1
        elif opt in ['-f', '--force']:
            force = 1
        elif opt in ['-i', '--init']:
            init_db = 1
        elif opt in ['-u', '--update']:
            update_db = 1
        elif opt in ['-h', '--help']:
            print usage_string
            sys.exit(1)
        else:
            print "Unknown Option '%s'" % opt
            print usage_string
            sys.exit(1)

    db_name = args[0]

    if init_db and update_db:
        print "Cannot init and update the DB"
        print usage_string
        sys.exit(-1)

    if init_db:
        if not force and raw_input("This will destroy all data in %s, are you sure(y/n)? " % db_name) not in ['y','Y']:
            sys.exit(0)
        import os
        from Ft.Ods.Tools import _4odb_destroy,_4odb_create
        try:
            _4odb_destroy.Run([1,db_name])
        except:
            pass
        _4odb_create.Run([1,db_name])

    parse((init_db or update_db),cpp,db_name,gen_pstubs,package,args=args,baseDir=directory)

if __name__ == '__main__':
    import sys
    Run(sys.argv)



