########################################################################
#
# File Name:      Dbm.py
#
# Documentation:  http://docs.4suite.org/4ODS/StorageManager/Adapters/Dbm.py.html
#
"""
The DBM database back-end adapter.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.org/COPYRIGHT  for license and copyright information
"""

import string, types, time, os, sys

from Ft.Ods import FtodsBindingException
from Ft.Ods import FtodsObjectNotFound
from Ft.Ods.StorageManager.Adapters import Constants

from Ft.Lib import DbmDatabase
from Ft.Lib import Set
from Ft.Ods.StorageManager.Adapters import ClassCache
import Manager

import tempfile

########################################
# NOTE:
# Interfaces in this class are those that every driver must support
#############################################


class DbAdapter:
    
    ################
    # Transactional Interface
    ################

    manager = Manager.DbManager()


    def begin(self,db):
        pass

    def commit(self,db):
        # Save changes
        db.commit()
        

    def checkpoint(self,db):
        #Save changes, but don't close
        db.checkpoint()

    def abort(self,db):
        # Ignore changes
        db.rollback()
        

    ############################
    ###Locking
    ############################

    def lock(self,repoId,mode):
        # Single user only, for now...
        pass

    def try_lock(self,repoId,mode):
        # Single user only, for now...
        pass


    #############################
    #MISC
    #############################
    def objectUptoDate(self,db,persistentType,id_,testTime):
        if persistentType == Constants.ObjectTypes.REPO:
            t = db['repos']
        elif persistentType == Constants.ObjectTypes.COLLECTION:
            t = db['collections']
        elif persistentType == Constants.ObjectTypes.OBJECT:
            t = db['objects']
        else:
            raise "Finish"
        if t.has_key(str(id_)):
            return t[str(id_)][2] <= testTime
        return 0
    


    ###########################
    #Python Classes
    ###########################
    def newPythonClass(self,db,klass):
        seq = db['sequences']
        pid = seq['classIds']
        seq['classIds'] = pid + 1
        modName = str(klass.__module__)
        if modName[:8] == '__main__':
            dirs = os.path.splitext(sys.argv[0])[0]
            modName = string.replace(dirs,'/','.') + modName[8:]
        classes = db['classes']
        classes[str(pid)] = (modName, klass.__name__)
        classes[str(hash((modName, klass.__name__)))] = pid
        return pid

    def getPythonClass(self,db,pid):
        classes = db['classes']
        return classes.get(str(pid),None)

    def deletePythonClass(self,db,pid):
        classes = db['classes']
        if classes.has_key(str(pid)):
            del classes[str(pid)]

    def getPythonClassId(self,db,klass):
        klassId = ClassCache.g_cache.getClassId(klass)
        if klassId is None:
            modName = str(klass.__module__)
            if modName[:8] == '__main__':
                dirs = os.path.splitext(sys.argv[0])[0]
                modName = string.replace(dirs,'/','.') + modName[8:]
            
            klassId = db['classes'].get(str(hash((modName, klass.__name__))))
        return klassId

    def _4ods_getPythonClassId(self,db,klass):
        cid = self.getPythonClassId(db,klass)
        if cid is None:
            raise "Unknown Class %s, did you register it?" % str(klass)
        return cid

    #####################
    #Repository Management
    #####################
    def newRepositoryObjectId(self,db,metaKind,klass,rid = None):
        if rid == None:
            seq = db['sequences']
            rid = seq['repoIds']
            seq['repoIds'] = rid + 1
        klassId = self._4ods_getPythonClassId(db,klass)
        repos = db['repos']
        repos[str(rid)] = [klassId,[None]*len(klass._tupleTypes),None]
        return rid


    #Repository Object Helpers
    def newInterfaceStorage(self,db,interface):
        pass


    def writeRepositoryObject(self,db,metaKind,rid,types,names,values):
        repos = db['repos']
        for ctr in range(len(values)):
            if values[ctr] is not None:
                repos[str(rid)][1][ctr] = values[ctr]
        repos[str(rid)][2] = time.time()
        return 1

    def getRepositoryObject(self,db,rid):
        repos = db['repos']
        data = repos.get(str(rid))
        if data is None:
            return data
        kid = int(data[0])
        klass = ClassCache.g_cache.getClass(self,db,kid)
        return (klass,data[1])

    def deleteRepositoryObject(self,db,rid,metaKind):
        repos = db['repos']
        if repos.has_key(str(rid)):
            del repos[str(rid)]


    #############################
    #Object Interface
    #############################
    def newObjectId(self,db,klass):
        seq = db['sequences']
        oid = seq['objIds']
        seq['objIds'] = oid + 1
        klassId = self._4ods_getPythonClassId(db,klass)
        objs = db['objects']
        objs[str(oid)] = [klassId,[None]*len(klass._tupleTypes),None]
        return oid

    def writeObject(self,db,oid,typeId,types,names,nameIds,values):
        objs = db['objects']
        for ctr in range(len(values)):
            if values[ctr] is not None:
                objs[str(oid)][1][ctr] = values[ctr]
        objs[str(oid)][2] = time.time()
        return 1

    def getObject(self,db,oid):
        objs = db['objects']
        data = objs.get(str(oid))
        if data is None:
            return data
        kid = int(data[0])
        klass = ClassCache.g_cache.getClass(self,db,kid)
        return (klass,data[1])


    def deleteObject(self,db,oid,klass):
        objs = db['objects']
        if objs.has_key(str(oid)):
            del objs[str(oid)]


    def getAllObjectIds(self,db):
        return map(int, db['objects'].keys()) 



    ###########################
    #Extent interface
    ###########################
    def addExtent(self,db,name,persistentType):
        bn = db['extents']
        bn[name] = (persistentType,[])

    def dropExtent(self,db,name):
        bn = db['extents']
        if bn.has_key(name):
            del bn[name]

    def addExtentEntry(self,db,names,oid):
        bn = db['extents']
        for name in names:
            bn[name][1].append(oid)

    def dropExtentEntry(self,db,names,oid):
        bn = db['extents']
        for name in names:
            if oid in bn[name][1]:
                bn[name][1].remove(oid)

    def extentNames(self,db):
        return db['extents'].keys()

    def loadExtent(self,db,name):
        return db['extents'].get(name,None)

    ###########################
    #Binding interface
    ###########################
    def bind(self,db,oid,name):
        bn = db['boundNames']
        bn[name] = oid

    def unbind(self,db,name):
        bn = db['boundNames']
        if bn.has_key(name):
            del bn[name]

    def getAllBindings(self,db):
        return db['boundNames'].keys()

    def getObjectBindings(self,db,oid):
        #This will be slow
        bindings = []
        for name,o in db['boundNames'].items():
            if oid == o:
                bindings.append(name)
        return bindings

    def lookup(self,db,name):
        return db['boundNames'].get(name,None)
        

    ############################
    #Collection Interface
    ###########################
    def newCollection(self,db,klass,collType,subType):
        seq = db['sequences']
        cid = seq['collectionIds']
        seq['collectionIds'] = cid + 1

        klassId = self._4ods_getPythonClassId(db,klass)
        cols = db['collections']
        cols[str(cid)] = [klassId,[],None,subType]
        return cid

    def writeCollection(self,db,cid,changes,collType,subType):

        col = db['collections'][str(cid)]

        for change in changes:
            if change[0] in [Constants.CollectionChanges.INSERT,Constants.CollectionChanges.APPEND]:
                values = change[1]
                for value,index in values:
                    if change[0] == Constants.CollectionChanges.INSERT:
                        #Shift all other indexs
                        col[1].insert(index,value)
                    else:
                        col[1].append(value)
            else:
                for index in change[1]:
                    del col[1][index]

        col[2] = time.time()
        return 1

    def getCollection(self,db,cid):
        col = db['collections'].get(str(cid))
        if not col:
            return None

        klass = ClassCache.g_cache.getClass(self,db,col[0])
        return (klass,col[3],col[1])

    def deleteCollection(self,db,cid,collType,subType):
        cols = db['collections']
        if cols.has_key(str(cid)):
            del cols[str(cid)]
        


    #######################################
    #Blobs
    #######################################
    def newBlob(self,db):
        seq = db['sequences']
        bid = seq['blobIds']
        seq['blobIds'] = bid + 1
        bs = db['blobs']
        tempfile.tempdir=DbmDatabase.DATABASE_DIR
        bs[str(bid)] = tempfile.mktemp()
        return bid

    def readBlob(self,db,bid):
        fName = db['blobs'].get(str(bid),None)
        if fName is None:
            raise IOError
        return open(fName).read()
    def writeBlob(self,db,bid,data):
        fName = db['blobs'].get(str(bid),None)
        if fName is None:
            raise IOError
        open(fName,'w').write(data)
    def deleteBlob(self,db,bid):
        bs = db['blobs']
        if bs.has_key(str(bid)):
            fName = db['blobs'].get(str(bid),None)
            os.unlink(fName)
            del db[str(bid)]


    #############################
    #Literal Interface
    #############################


    #Garbage Collection
    def collectGarbage(self):
        self._4ods_collectRepositoryObjectGarbage()



