class UserAccessLevel:
    ALL = 0
    METADATA = 1
    OBJECT = 2


class DbManager:
    """
    Abstract base class for all database managers
    """

    def connect(self, connectString):
        raise NotImplementedError("Subclass must override")


    def close(self, conn):
        raise NotImplementedError("Subclass must override")
    

    def clean(self, connectString):
        """
        Remove all objects from the database that are not accessible from either
        an extent or a bound name.  In other words, garbage collection.
        """
        raise NotImplementedError("Subclass must override")


    def clear(self, connectString):
        """
        Remove all existing data from the specified database.
        """
        raise NotImplementedError("Subclass must override")


    def create(self, connectString):
        """
        Create a new database ready to accept ODL.
        """
        raise NotImplementedError("Subclass must override")


    def destory(self, connectString):
        """
        Completely removes the database specified by the connectString
        from the system.
        """
        raise NotImplementedError("Subclass must override")


    def grant(self, connectString, username, accessLevel):
        """
        Changes the access level of the given user in the specified database.
        """
        raise NotImplementedError("Subclass must override")

    
    def exists(self, connectString):
        raise NotImplementedError("Subclass must override")

    
    def init(self, connectString):
        raise NotImplementedError("Subclass must override")


    def initRepo(self, connectString, adapter):
        raise NotImplementedError("Subclass must override")


    def reinit(self, connectString, adapter):
        raise NotImplementedError("Subclass must override")


    def new(self, connectString, adapter):
        self.create(connectString)
        self.init(connectString)
        self.initRepo(connectString, adapter)
        return


    def reinitRepo(self, connectString, adapter):
        self.reinit(connectString, adapter)
        self.initRepo(connectString, adapter)
        return
