########################################################################
#
# File Name: 	        Postgres.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/Postgres.py.html
#
"""
The Postgres back-end.
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""
import sys,string,types

import string, threading, cPickle
from Ft.Ods import FtodsBindingException
from Ft.Ods import FtodsObjectNotFound

from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.StorageManager.Adapters import SQL
from Ft.Ods.StorageManager.Adapters.SQL import CompiliedStatement
import PostgresStatements
import Manager
import _pg

SIZE_STRING_LEN=20

def _4ods_anyToOdmg(value):
    return cPickle.loads(value)

def _4ods_anyToSql(value):
    return "'"+escapeQuotes(cPickle.dumps(value))+"'"

def escapeQuotes(qstr):
    """----------------------------------------------------------
    Postgres uses single quotes for string marker, so put a
    backslash before single quotes for insertion into a database.
    pre: qstr = string to be escaped
    post: return the string with all single quotes escaped
    ----------------------------------------------------------"""
    if qstr is None:
        return ''
    tmp = string.replace(qstr,"\\","\\\\")
    tmp = string.replace(tmp, "'", "\\'")
    return tmp






class DbAdapter(SQL.DbAdapter):
    statements = {}
    uptoDateStatements = {}
    nameMap = {}
    manager = Manager.DbManager()
    
    ################
    #The Transactional Interface
    ################
    def begin(self,db):
        db.query('BEGIN')


    def commit(self,db):
        db.query('COMMIT')
        db.close()

    def abort(self,db):
        db.close()

    def checkpoint(self,db):
        db.query('COMMIT')
        db.query('BEGIN')
        

    def lock(self,db,repoId,mode):
        raise "NOT DONE"
        table_name = self._4ods_getTableNameFromRepoId(txId,repoId)
        if mode == Constants.READ_LOCK:
            #Do a read lock
            qstr = 'LOCK %s IN ACCESS SHARE MODE'
        else:
            #Do a write lock
            qstr = 'LOCK %s in ACCESS EXCLUSIVE MODE'

        self._db.execute(qstr)
        return

    def try_lock(self,repoId,mode):
        raise "NOT DONE"
        #FIXME how do we do this in Postgres???
        self.lock(repoId,mode)



    #####################################
    #MISC
    #####################################


    def _execute(self,db,st):
        return db.query(st)

    statements[CompiliedStatement.NEW_PYTHON_CLASS_ID] = PostgresStatements.NewPythonClassIdStatement()
    statements[CompiliedStatement.NEW_PYTHON_CLASS] = PostgresStatements.NewPythonClassStatement()
    statements[CompiliedStatement.GET_PYTHON_CLASS] = PostgresStatements.GetPythonClassStatement()
    statements[CompiliedStatement.DELETE_PYTHON_CLASS] = PostgresStatements.DeletePythonClassStatement()
    statements[CompiliedStatement.GET_PYTHON_CLASS_ID] = PostgresStatements.GetPythonClassIdStatement()
    

    #####################
    #Repository Management
    #####################
    statements[CompiliedStatement.NEW_REPO_ID] = PostgresStatements.NewRepoIdStatement()
    statements[CompiliedStatement.NEW_REPO] = PostgresStatements.NewRepoStatement()
    statements[CompiliedStatement.NEW_REPO_MAP] = PostgresStatements.NewRepoMapStatement()
    statements[CompiliedStatement.GET_REPO] = PostgresStatements.GetRepoStatement()
    statements[CompiliedStatement.GET_REPO_TUPLE] = PostgresStatements.GetRepoTupleStatement()
    statements[CompiliedStatement.DELETE_REPO] = PostgresStatements.DeleteRepoStatement()
    statements[CompiliedStatement.DELETE_REPO_MAP] = PostgresStatements.DeleteRepoMapStatement()
    statements[CompiliedStatement.UPDATE_REPO_TIME] = PostgresStatements.UpdateRepoTimeStatement()
    uptoDateStatements[Constants.ObjectTypes.REPO] = PostgresStatements.CheckRepoTimeStatement()
    nameMap['references'] = '_references'


    #################################
    #Objects
    #################################
    statements[CompiliedStatement.NEW_OBJECT_ID] = PostgresStatements.NewObjectIdStatement()
    statements[CompiliedStatement.NEW_OBJECT_MAP] = PostgresStatements.NewObjectMapStatement()
    statements[CompiliedStatement.NEW_OBJECT] = PostgresStatements.NewObjectStatement()
    statements[CompiliedStatement.UPDATE_OBJECT_TIME] = PostgresStatements.UpdateObjectTimeStatement()
    statements[CompiliedStatement.GET_OBJECT] = PostgresStatements.GetObjectStatement()
    statements[CompiliedStatement.GET_OBJECT_TUPLE] = PostgresStatements.GetObjectTupleStatement()
    statements[CompiliedStatement.DELETE_OBJECT] = PostgresStatements.DeleteObjectStatement()
    statements[CompiliedStatement.DELETE_OBJECT_MAP] = PostgresStatements.DeleteObjectMapStatement()
    statements[CompiliedStatement.GET_OBJECT_IDS] = PostgresStatements.GetObjectIdsStatement()
    uptoDateStatements[Constants.ObjectTypes.OBJECT] = PostgresStatements.CheckObjectTimeStatement()


    #############################
    #Extents
    #############################
    statements[CompiliedStatement.NEW_EXTENT_ID] = PostgresStatements.NewExtentIdStatement()
    statements[CompiliedStatement.NEW_EXTENT] = PostgresStatements.NewExtentStatement()
    statements[CompiliedStatement.GET_EXTENT_ID] = PostgresStatements.GetExtentIdStatement()
    statements[CompiliedStatement.ADD_EXTENT_MAPPING] = PostgresStatements.AddExtentMappingStatement()
    statements[CompiliedStatement.DROP_EXTENT_MAPPING] = PostgresStatements.DropExtentMappingStatement()
    statements[CompiliedStatement.DROP_ALL_EXTENT_MAPPING] = PostgresStatements.DropAllExtentMappingStatement()
    statements[CompiliedStatement.DROP_EXTENT] = PostgresStatements.DropExtentStatement()
    statements[CompiliedStatement.GET_EXTENT_NAMES] = PostgresStatements.GetExtentNamesStatement()
    statements[CompiliedStatement.GET_EXTENT_ID_AND_TYPE] = PostgresStatements.GetExtentIdAndTypeStatement()
    statements[CompiliedStatement.GET_EXTENT] = PostgresStatements.GetExtentStatement()


    ##############################
    #Bindings
    #############################
    statements[CompiliedStatement.GET_BOUND_OBJECT] = PostgresStatements.GetBoundObjectStatement()
    statements[CompiliedStatement.ADD_BOUND_NAME] = PostgresStatements.AddBoundNameStatement()
    statements[CompiliedStatement.DROP_BOUND_NAME] = PostgresStatements.DropBoundNameStatement()
    statements[CompiliedStatement.GET_BOUND_NAMES] = PostgresStatements.GetBoundNamesStatement()
    statements[CompiliedStatement.GET_OBJECT_BOUND_NAMES] = PostgresStatements.GetObjectBoundNamesStatement()


    ##############################
    #Collections
    ##############################
    statements[CompiliedStatement.NEW_COLLECTION_ID] = PostgresStatements.NewCollectionIdStatement()
    statements[CompiliedStatement.NEW_COLLECTION] = PostgresStatements.NewCollectionStatement()
    statements[CompiliedStatement.UPDATE_COLLECTION_INDEXES_ADD] = PostgresStatements.UpdateCollectionIndexesAddStatement()
    statements[CompiliedStatement.NEW_COLLECTION_ENTRY] = PostgresStatements.NewCollectionEntryStatement()
    statements[CompiliedStatement.DELETE_COLLECTION_ENTRY] = PostgresStatements.DeleteCollectionEntryStatement()
    statements[CompiliedStatement.UPDATE_COLLECTION_INDEXES_REMOVE] = PostgresStatements.UpdateCollectionIndexesRemoveStatement()
    statements[CompiliedStatement.DELETE_COLLECTION] = PostgresStatements.DeleteCollectionStatement()
    statements[CompiliedStatement.DELETE_COLLECTION_TUPLE] = PostgresStatements.DeleteCollectionTupleStatement()
    statements[CompiliedStatement.GET_COLLECTION] = PostgresStatements.GetCollectionStatement()
    statements[CompiliedStatement.GET_COLLECTION_TUPLE] = PostgresStatements.GetCollectionTupleStatement()
    statements[CompiliedStatement.UPDATE_COLLECTION_TIME] = PostgresStatements.UpdateCollectionTimeStatement()

    uptoDateStatements[Constants.ObjectTypes.COLLECTION] = PostgresStatements.CheckCollectionTimeStatement()


    #######################
    #Blobs
    #######################
    #FIXME extra space is not released

    def newBlob(self,db):
        boid = db.locreate(_pg.INV_WRITE | _pg.INV_READ).oid
        db.query("INSERT into ftods_blobs (bid,boid) VALUES (%d,%d)" % (boid,boid))
        return boid

    def readBlob(self,db,bid):
        rt = db.query("SELECT boid from ftods_blobs WHERE bid = %d" % (bid)).getresult()
        if not len(rt):
            return None
        boid = int(rt[0][0])
        l = db.getlo(boid)
        try:
            l.open(_pg.INV_READ)
        except IOError:
            return None
        data = l.read(l.size())
        l.close()
        return eval(data)

    def writeBlob(self,db,bid,data):
        rt = db.query("SELECT boid from ftods_blobs WHERE bid = %d" % (bid)).getresult()
        if not len(rt):
            return None

        #Delete the old blob
        oldBoid = int(rt[0][0])
        l = db.getlo(oldBoid)
        l.unlink()


        #Create a new one
        boid = db.locreate(_pg.INV_WRITE | _pg.INV_READ).oid
        db.query("UPDATE ftods_blobs SET boid  = %d WHERE bid=%d" % (boid,bid))
        
        l = db.getlo(boid)
        l.open(_pg.INV_WRITE)
        l.write(repr(data))
        l.close()

    def deleteBlob(self,db,bid):
        rt = db.query("SELECT boid from ftods_blobs WHERE bid = %d" % (bid)).getresult()
        if not len(rt):
            return None
        boid = int(rt[0][0])
        l = db.getlo(boid)
        l.unlink()
        db.query("DELETE from ftods_blobs WHERE bid=%d" % (bid))
        




    #Overdides
    def _4ods_createUpdateObjectString(self,oid,typeId,types,names,nameIds,values):
        qstr = 'UPDATE interface__%s SET ' % typeId
        for ctr in range(len(nameIds)):
            if values[ctr] != None:
                if names[ctr] == '_oid':
                    continue
                qstr = qstr + 'def__%d = %s,' % (nameIds[ctr],self.toSqlValues[types[ctr]](values[ctr]))
                

        qstr = "%s WHERE oid = %i"%(qstr[:-1],oid)
        return qstr



    toSqlValues = {Constants.Types.STRING:lambda x:"'%s'"%escapeQuotes(x),
                   Constants.Types.BLOB:str,
                   Constants.Types.UNSIGNED_SHORT:str,
                   Constants.Types.OBJECT:str,
                   Constants.Types.ENUMERATION:str,
                   Constants.Types.SIGNED_SHORT:str,
                   Constants.Types.DOUBLE:str,
                   Constants.Types.FLOAT:str,
                   Constants.Types.UNSIGNED_LONG:lambda x:"%g"%x,
                   Constants.Types.SIGNED_LONG_LONG:lambda x:repr(long(x))[:-1],
                   Constants.Types.SIGNED_LONG:lambda x:"%g"%x,
                   Constants.Types.BOOLEAN:lambda x:x and "'t'" or "'f'",
                   Constants.Types.OBJECT_LIST:str,
                   Constants.Types.OBJECT_SET:str,
                   Constants.Types.OBJECT_BAG:str,
                   Constants.Types.STRING_LIST:str,
                   Constants.Types.STRING_BAG:str,
                   Constants.Types.STRING_SET:str,
                   Constants.Types.BOOLEAN_LIST:str,
                   Constants.Types.BOOLEAN_BAG:str,
                   Constants.Types.BOOLEAN_SET:str,
                   Constants.Types.SIGNED_SHORT_LIST:str,
                   Constants.Types.SIGNED_SHORT_BAG:str,
                   Constants.Types.SIGNED_SHORT_SET:str,
                   Constants.Types.UNSIGNED_SHORT_LIST:str,
                   Constants.Types.UNSIGNED_SHORT_BAG:str,
                   Constants.Types.UNSIGNED_SHORT_SET:str,
                   Constants.Types.SIGNED_LONG_LIST:str,
                   Constants.Types.SIGNED_LONG_BAG:str,
                   Constants.Types.SIGNED_LONG_SET:str,
                   Constants.Types.UNSIGNED_LONG_LIST:str,
                   Constants.Types.UNSIGNED_LONG_BAG:str,
                   Constants.Types.UNSIGNED_LONG_SET:str,
                   Constants.Types.SIGNED_LONG_LONG_LIST:str,
                   Constants.Types.SIGNED_LONG_LONG_BAG:str,
                   Constants.Types.SIGNED_LONG_LONG_SET:str,
                   Constants.Types.FLOAT_LIST:str,
                   Constants.Types.FLOAT_BAG:str,
                   Constants.Types.FLOAT_SET:str,
                   Constants.Types.DOUBLE_LIST:str,
                   Constants.Types.DOUBLE_BAG:str,
                   Constants.Types.DOUBLE_SET:str,
                   Constants.Types.ANY:_4ods_anyToSql,
                   }
                   

    toOdmgValues = {Constants.Types.UNSIGNED_SHORT:int,
                    Constants.Types.OBJECT:int,
                    Constants.Types.ENUMERATION:int,
                    Constants.Types.SIGNED_SHORT:int,
                    Constants.Types.UNSIGNED_LONG:int,
                    Constants.Types.SIGNED_LONG_LONG:long,
                    Constants.Types.SIGNED_LONG:long,
                    Constants.Types.DOUBLE:float,
                    Constants.Types.FLOAT:float,
                    Constants.Types.STRING:str,
                    Constants.Types.BLOB:int,
                    Constants.Types.BOOLEAN:lambda x: x == 't',
                    Constants.Types.OBJECT_LIST:int,
                    Constants.Types.OBJECT_SET:int,
                    Constants.Types.OBJECT_BAG:int,
                    Constants.Types.STRING_LIST:int,
                    Constants.Types.STRING_BAG:int,
                    Constants.Types.STRING_SET:int,
                    Constants.Types.BOOLEAN_LIST:int,
                    Constants.Types.BOOLEAN_BAG:int,
                    Constants.Types.BOOLEAN_SET:int,
                    Constants.Types.SIGNED_SHORT_LIST:int,
                    Constants.Types.SIGNED_SHORT_BAG:int,
                    Constants.Types.SIGNED_SHORT_SET:int,
                    Constants.Types.UNSIGNED_SHORT_LIST:int,
                    Constants.Types.UNSIGNED_SHORT_BAG:int,
                    Constants.Types.UNSIGNED_SHORT_SET:int,
                    Constants.Types.SIGNED_LONG_LIST:int,
                    Constants.Types.SIGNED_LONG_BAG:int,
                    Constants.Types.SIGNED_LONG_SET:int,
                    Constants.Types.UNSIGNED_LONG_LIST:int,
                    Constants.Types.UNSIGNED_LONG_BAG:int,
                    Constants.Types.UNSIGNED_LONG_SET:int,
                    Constants.Types.SIGNED_LONG_LONG_LIST:int,
                    Constants.Types.SIGNED_LONG_LONG_BAG:int,
                    Constants.Types.SIGNED_LONG_LONG_SET:int,
                    Constants.Types.FLOAT_LIST:int,
                    Constants.Types.FLOAT_BAG:int,
                    Constants.Types.FLOAT_SET:int,
                    Constants.Types.DOUBLE_LIST:int,
                    Constants.Types.DOUBLE_BAG:int,
                    Constants.Types.DOUBLE_SET:int,
                    Constants.Types.ANY:_4ods_anyToOdmg,
                    }




