########################################################################
#
# File Name: 	        SQL.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Drivers/Postgres.py.html
#
"""
Common backend for all SQL'ish databases
WWW: http://4suite.com/4ODS         e-mail: support@4suite.com

Copyright (c) 1999 FourThought, Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""
import sys,string,types, os

import string, threading, time
from Ft.Ods import FtodsBindingException
from Ft.Ods import FtodsObjectNotFound

from Ft.Ods.StorageManager.Adapters import Constants
from Ft.Ods.StorageManager.Adapters import ClassCache

import CompiliedStatement

class DbAdapter:
    statements = {}
    uptoDateStatements = {}
    nameMap = {}

    ################
    #The Transactional Interface

    ################
    def begin(self,db):
        raise "Override"

    def commit(self,db):
        raise "Override"

    def abort(self,db):
        raise "Override"

    def checkpoint(self,db):
        raise "Override"
        

    ############################
    ###Locking
    ############################

    def lock(self,db,repoId,mode):
        raise "Override"


    def try_lock(self,db,repoId,mode):
        raise "Override"

    #############################
    #MISC
    #############################
    def _execute(self,db,st):
        raise "Override"

    def objectUptoDate(self,db,persistentType,id_,testTime):
        rt = self.uptoDateStatements[persistentType].query(db,id=id_,testTime=testTime)
        return len(rt)

    ###########################
    #Python Classes
    ###########################
    def newPythonClass(self,db,klass):
        pid = int(self.statements[CompiliedStatement.NEW_PYTHON_CLASS_ID].query(db)[0][0])
        modName = str(klass.__module__)
        if modName[:8] == '__main__':
            dirs = os.path.splitext(sys.argv[0])[0]
            modName = string.replace(dirs,'/','.') + modName[8:]
        self.statements[CompiliedStatement.NEW_PYTHON_CLASS].execute(db,pid=pid,moduleName=modName, className=klass.__name__)
        return pid

    def getPythonClass(self,db,pid):
        rt = self.statements[CompiliedStatement.GET_PYTHON_CLASS].query(db,pid=pid)
        if len(rt) == 0:
            return None
        return rt[0][0],rt[0][1]

    def deletePythonClass(self,db,pid):
        self.statements[CompiliedStatement.DELETE_PYTHON_CLASS].execute(db,pid=pid)


    def getPythonClassId(self,db,klass):
        #if hasattr(klass,'_4ods_klassId'):
        #    return klass._4ods_klassId
        klassId = ClassCache.g_cache.getClassId(klass)
        if klassId is None:
            modName = str(klass.__module__)
            if modName[:8] == '__main__':
                dirs = os.path.splitext(sys.argv[0])[0]
                modName = string.replace(dirs,'/','.') + modName[8:]
            klassId = self.statements[CompiliedStatement.GET_PYTHON_CLASS_ID].query(db,moduleName=modName, className=klass.__name__)
            if not len(klassId):
                return None
            klassId = int(klassId[0][0])
            ClassCache.g_cache.registerClassId(klassId,klass)
        #klass._4ods_klassId = klassId
        return klassId

    def _4ods_getPythonClassId(self,db,klass):
        cid = self.getPythonClassId(db,klass)
        if cid is None:
            raise "Unknown Class %s, did you register it?" % str(klass)
        return cid

    #####################
    #Repository Management
    #####################
    def newRepositoryObjectId(self,db,metaKind,klass,rid = None):
        if rid == None:
            rid = int(self.statements[CompiliedStatement.NEW_REPO_ID].query(db)[0][0])
        if metaKind == 2 and 'key_type' in klass._tupleNames:
            tableName = 'ftods_dictionary'
        else:
            tableName = self.manager.metaKindToTableName[metaKind]
        klassId = self._4ods_getPythonClassId(db,klass)
        self.statements[CompiliedStatement.NEW_REPO].execute(db,tableName=tableName,rid=rid)
        self.statements[CompiliedStatement.NEW_REPO_MAP].execute(db,metaKind=metaKind,rid=rid,pythonclassid = klassId)
        return rid


    #Repository Object Helpers
    def newInterfaceStorage(self,db,interface):
        data = self._buildInterfaceTable(interface)
        self.manager._createInterfaceStorage(db,data)


    def writeRepositoryObject(self,db,metaKind,rid,types,names,values):
        if metaKind == 2 and 'key_type' in names:
            tableName = 'ftods_dictionary'
        else:
            tableName = self.manager.metaKindToTableName[metaKind]

        names = map(lambda x,n=self.nameMap:n.get(x,x),names)
        qstr = self._4ods_createUpdateRepoString(tableName,rid,types,names,values)
        if not qstr: return
        self._execute(db,qstr)
        rt = self.statements[CompiliedStatement.UPDATE_REPO_TIME].execute(db,rid=rid,newTime=time.time())
        return 1

    def getRepositoryObject(self,db,id):

        rt = self.statements[CompiliedStatement.GET_REPO].query(db,rid=id)
        if len(rt) == 0:
            return None
        metakind = int(rt[0][0])
        kid = int(rt[0][1])
        klass = ClassCache.g_cache.getClass(self,db,kid)
        if metakind == 2 and 'key_type' in klass._tupleNames:
            tableName = 'ftods_dictionary'
        else:
            tableName = self.manager.metaKindToTableName[metakind]

        #Now load the tuple
        tup = self.statements[CompiliedStatement.GET_REPO_TUPLE].query(db,tableName=tableName,rid=id)[0]
        return (klass,map(lambda t,v,o=self.toOdmgValues:o[t](v),klass._tupleTypes,tup))


    def deleteRepositoryObject(self,db,rid,metaKind):
        tableName = self.manager.metaKindToTableName[metaKind]
        self.statements[CompiliedStatement.DELETE_REPO].execute(db,tableName=tableName,rid = rid)
        self.statements[CompiliedStatement.DELETE_REPO_MAP].execute(db,rid = rid)
        if metaKind == 2:
            self.statements[CompiliedStatement.DELETE_REPO].execute(db,tableName='ftods_dictionary',rid = rid)


    #############################
    #Object Interface
    #############################
    def newObjectId(self,db,klass):
        tableName = 'interface__%d' % klass._typeId
        oid = int(self.statements[CompiliedStatement.NEW_OBJECT_ID].query(db,tableName=tableName)[0][0])
        klassId = self._4ods_getPythonClassId(db,klass)
        self.statements[CompiliedStatement.NEW_OBJECT_MAP].execute(db,oid=oid,pythonclassid = klassId)
        self.statements[CompiliedStatement.NEW_OBJECT].execute(db,tableName=tableName,oid=oid)
        return oid

    def writeObject(self,db,oid,typeId,types,names,nameIds,values):
        # Remove the _oid stuff from the tuples
        qstr = self._4ods_createUpdateObjectString(oid,typeId,types[1:],names[1:],nameIds[1:],values[1:])
        if not qstr: return
        self._execute(db,qstr)
        rt = self.statements[CompiliedStatement.UPDATE_OBJECT_TIME].execute(db,oid=oid,newTime=time.time())
        return 1
        
    def getObject(self,db,oid):
        rt = self.statements[CompiliedStatement.GET_OBJECT].query(db,oid=oid)
        if len(rt) == 0:
            return None
        kid = int(rt[0][0])
        klass = ClassCache.g_cache.getClass(self,db,kid)
        tableName = 'interface__%d' % klass._typeId

        # Now load the tuple
        stmt = self.statements[CompiliedStatement.GET_OBJECT_TUPLE]
        # Add the oid to the results
        results = stmt.query(db,tableName=tableName,oid=oid)
        if not results:
            return None
        return (klass,map(lambda t,v,o=self.toOdmgValues:o[t](v),klass._tupleTypes,results[0]))

    def deleteObject(self,db,oid,klass):
        tableName = 'interface__%d' % klass._typeId
        self.statements[CompiliedStatement.DELETE_OBJECT].execute(db,tableName=tableName,oid = oid)
        self.statements[CompiliedStatement.DELETE_OBJECT_MAP].execute(db,oid = oid)

    def getAllObjectIds(self,db):
        return map(lambda x:int(x[0]),self.statements[CompiliedStatement.GET_OBJECT_IDS].query(db))




    ###########################
    #Extent interface
    ###########################
    def addExtent(self,db,name,persistentType):
        id = self.statements[CompiliedStatement.NEW_EXTENT_ID].query(db)[0][0]
        self.statements[CompiliedStatement.NEW_EXTENT].execute(db,name=name,extentId=id,persistentType=persistentType)

    def dropExtent(self,db,name):
        rt = self.statements[CompiliedStatement.GET_EXTENT_ID].query(db,name=name)
        self.statements[CompiliedStatement.DROP_ALL_EXTENT_MAPPING].execute(db,extentId=int(rt[0][0]))
        self.statements[CompiliedStatement.DROP_EXTENT].execute(db,extentId=int(rt[0][0]))

    def addExtentEntry(self,db,names,oid):
        for name in names:
            rt = self.statements[CompiliedStatement.GET_EXTENT_ID].query(db,name=name)
            self.statements[CompiliedStatement.ADD_EXTENT_MAPPING].execute(db,extentId=int(rt[0][0]),id=oid)

    def dropExtentEntry(self,db,names,oid):
        for name in names:
            rt = self.statements[CompiliedStatement.GET_EXTENT_ID].query(db,name=name)
            self.statements[CompiliedStatement.DROP_EXTENT_MAPPING].execute(db,extentId=int(rt[0][0]),id=oid)

    def extentNames(self,db):
        rt = self.statements[CompiliedStatement.GET_EXTENT_NAMES].query(db)
        return map(lambda x:x[0],rt)

    def loadExtent(self,db,name):
        rt = self.statements[CompiliedStatement.GET_EXTENT_ID_AND_TYPE].query(db,name=name)
        if not rt:
            return None
        res = self.statements[CompiliedStatement.GET_EXTENT].query(db,extentId=int(rt[0][0]))
        return (int(rt[0][1]),map(lambda x:int(x[0]),res))
        


    ###########################
    #Binding interface
    ###########################
    def bind(self,db,oid,name):
        self.statements[CompiliedStatement.ADD_BOUND_NAME].execute(db,name=name,oid=oid)

    def unbind(self,db,name):
        #Make sure this name exists in the bindings
        self.statements[CompiliedStatement.DROP_BOUND_NAME].execute(db,name=name)


    def getAllBindings(self,db):
        rt = self.statements[CompiliedStatement.GET_BOUND_NAMES].query(db)
        return map(lambda x:x[0],rt)

    def getObjectBindings(self,db,oid):
        rt = self.statements[CompiliedStatement.GET_OBJECT_BOUND_NAMES].query(db,oid=oid)
        return map(lambda x:x[0],rt)

    def lookup(self,db,name):
        rt = self.statements[CompiliedStatement.GET_BOUND_OBJECT].query(db,name=name)
        if len(rt):
            return int(rt[0][0])
        return None

        

    ############################
    #Collection Interface
    ###########################
    def newCollection(self,db,klass,collType,subType):
        cid = int(self.statements[CompiliedStatement.NEW_COLLECTION_ID].query(db)[0][0])
        #Store its class
        klassId = self._4ods_getPythonClassId(db,klass)
        self.statements[CompiliedStatement.NEW_COLLECTION].execute(db,cid=cid,pythonclassid=klassId,subType=subType)
        return cid

    def writeCollection(self,db,cid,changes,collType,subType):

        tableName = self.manager.collectionTableMapping[subType][0]

        for change in changes:
            if change[0] in [Constants.CollectionChanges.INSERT,Constants.CollectionChanges.APPEND]:
                values = change[1]
                for value,index in values:
                    value = self.toSqlValues[subType](value)
                    if change[0] == Constants.CollectionChanges.INSERT:
                        #Shift all other indexs
                        self.statements[CompiliedStatement.UPDATE_COLLECTION_INDEXES_ADD].execute(db,cid=cid,tableName=tableName,index=index)
                    self.statements[CompiliedStatement.NEW_COLLECTION_ENTRY].execute(db,cid=cid,tableName=tableName,index=index,value=value)
            else:
                #Must be a remove
                for index in change[1]:
                    self.statements[CompiliedStatement.DELETE_COLLECTION_ENTRY].execute(db,cid=cid,tableName=tableName,index=index)
                    self.statements[CompiliedStatement.UPDATE_COLLECTION_INDEXES_REMOVE].execute(db,cid=cid,tableName=tableName,index=index)
        rt = self.statements[CompiliedStatement.UPDATE_COLLECTION_TIME].execute(db,cid=cid,newTime=time.time())

        return 1

    def getCollection(self,db,cid):
        rt = self.statements[CompiliedStatement.GET_COLLECTION].query(db,cid=cid)
        if len(rt) == 0:
            return None
        subType = int(rt[0][0])
        kid = int(rt[0][1])
        klass = ClassCache.g_cache.getClass(self,db,kid)
        tableName = self.manager.collectionTableMapping[subType][0]
        tup = self.statements[CompiliedStatement.GET_COLLECTION_TUPLE].query(db,cid=cid,tableName=tableName)
        return (klass,subType,map(lambda v,o=self.toOdmgValues[subType]:o(v[0]),tup))

    def deleteCollection(self,db,cid,collType,subType):
        tableName = self.manager.collectionTableMapping[subType][0]
        self.statements[CompiliedStatement.DELETE_COLLECTION].execute(db,cid=cid)
        self.statements[CompiliedStatement.DELETE_COLLECTION_TUPLE].execute(db,tableName=tableName,cid=cid)
        
        


    #######################################
    #Blobs
    #######################################
    def newBlob(self,db):
        raise "Please Override"
    def readBlob(self,db,bid):
        raise "Please Override"
    def writeBlob(self,db,oid,data):
        raise "Please Overide"
    def deleteBlob(self,db,oid):
        raise "Please Override"
    



    #############################
    #Literal Interface
    #############################


    #Garbage Collection
    def collectGarbage(self):
        self._4ods_collectRepositoryObjectGarbage()





    ##########################
    #Internal Interfaces
    ##########################
    def _4ods_createUpdateRepoString(self,tableName,rid,types,names,values):
        qstr = self._4ods_tupleToUpdateString(types,names,values)
        if not qstr: return
        return 'UPDATE %s SET %s WHERE _repoid = %i' % (tableName,
                                                        qstr,
                                                        rid
                                                        )

    def _4ods_createUpdateObjectString(self,oid,typeId,types,names,nameIds,values):
        newNames = map(lambda x:'def__%d'%x,nameIds)
        qstr = self._4ods_tupleToUpdateString(types,newNames,values)
        if not qstr: return
        tableName = 'interface__%i' % typeId
        qstr = 'UPDATE %s SET %s WHERE oid = %i' % (tableName,
                                                    qstr,
                                                    oid
                                                    )
        return qstr

    def _4ods_tupleToUpdateString(self,types,names,values):
        qstr = ''
        for ctr in range(len(names)):
            if values[ctr] != None:
                if names[ctr] == '_oid':
                    continue
                qstr = qstr + names[ctr] + '=' + self.toSqlValues[types[ctr]](values[ctr]) + ','

        return qstr[:-1]


    def _buildInterfaceTable(self,interface,foundNames = None):
        foundNames = foundNames or []
        tableName = "interface__%i"%interface._4ods_getId()
        if tableName in foundNames: return None
        foundNames.append(tableName)
        defs = []
        for d in interface.defines:
            if d.meta_kind._v == 12:
                #A refernece to some python code?????
                #raise "We don;t yet know what to do with operations!!!!!!"
                pass
            elif d.meta_kind._v in [0,15]:
                #We don't do any thing with structures here
                defs.append((d._4ods_getId(),self.manager.odmgToSqlTypes[d.type._4ods_getOdmgType()],''))
        

        bClasses = []
        for inh in interface.inherits:
            rt = self._buildInterfaceTable(inh,foundNames)
            if rt:
                bClasses.append(rt)
        if interface.meta_kind._v == 1 and interface.extender:
            rt = self._buildInterfaceTable(interface.extender,foundNames)
            if rt:
                bClasses.append(rt)
        if len(bClasses) == 0 and 'ftods_object' not in foundNames:
            data = map(lambda x,o=self.manager.odmgToSqlTypes:(x[0],o[x[1]],x[2]),self.manager.tables['ftods_object'][self.manager.TableData.TABLE_INIT])
            bClasses.append(('ftods_object',data,[]))
            foundNames.append('ftods_object')
        return (tableName,defs,bClasses)


