############################################################################
#
# File Name: 		ExtentManager.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/StorageManager/ExtentManager.py/html
#

#Manages Extents
#WWW: http://4suite.com/4ODS         e-mail: support@4suite.com
#
#Copyright (c) 2000 Fourthought Inc, USA.   All Rights Reserved.
#See  http://4suite.com/COPYRIGHT  for license and copyright information


#NOTE Do not inherit from CollectionBase becuase this is a readonly collection (can be rewritten much simplier)

from Ft.Ods.StorageManager.Adapters import Constants


class Extent:
    def __init__(self,name,manager,data=None):
        self._name = name
        if data is None:
            self._new = 1
            self.data = []
            self.persistentType = Constants.ObjectTypes.OBJECT
        else:
            self.persistentType = data[0]
            self.data = list(data[1])
            self._new = 0

        self.manager = manager
        self._addedObjects = []
        self._deletedObjectIds = []
        
        self._delete = 0

    def cardinality(self):
        return len(self.data)

    def is_empty(self):
        return self.cardinality() == 0

    def is_ordered(self):
        return 0

    def allows_duplicates(self):
        return 0

    def contains_element(self, e):
        raise "FIXME"

    def create_iterator(self, stable):
        return Iterator.Iterator(self, stable)

    def create_bidirectional_iterator(self, stable):
        return BidirectionalIterator.BidirectionalIterator(self, stable)

    def select_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def select(self,OQL_predicate):
        raise FtodsNotSupportedException("select")

    def query(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")

    def exists_element(self,OQL_predicate):
        raise FtodsNotSupportedException("select_element")


    #The pythonic interface
    def __len__(self): return self.cardinality()
    def __getitem__(self, i):
        if i >= self.cardinality():
            raise IndexError(i)
        if i < 0:
            i = self.cardinality() + i
        thing = self.data[i]
        if type(thing) == type(1):
            if self.persistentType == Constants.ObjectTypes.REPO:
                obj = self.manager._4ods_getRepositoryObject(thing)
            else:
                obj = self.manager._4ods_getObject(thing)
            self.data[i] = obj
            return obj
        return thing

    def __getslice__(self, i, j):
        i = max(i, 0); j = min(j, self.cardinality())
        res = []
        for ctr in range(i,j):
            res.append(self[ctr])
        return res

    def pop(self, i=-1): raise "FIXME"
    def count(self, item): raise "FIXME"
    def index(self, item): raise "FIXME"


    def _4ods_delete(self):
        self._delete = 1

    def _4ods_addObject(self,object):
        self._addedObjects.append(object)
        self.data.append(object)
        if self.persistentType == None:
            self.persistentType = object.presistentType

    def _4ods_removeObject(self,object):
        if object._4ods_getId():
            self._deletedObjectIds.append(object._4ods_getId())
        else:
            self._addedObjects.remove(object)
        cmp = (object,object._4ods_getId())
        for ctr in range(len(self.data)):
            if self.data[ctr] in cmp:
                del self.data[ctr]
                break

            

    def _4ods_updateChanges(self,adapter,db):

        if self._delete:
            adapter.dropExtent(db,self._name)
        else:
            if self._new:
                adapter.addExtent(db,self._name,self.persistentType)


            for obj in self._addedObjects:
                adapter.addExtentEntry(db,[self._name],obj._4ods_getId())

            for objId in self._deletedObjectIds:
                adapter.dropExtentEntry(db,[self._name],objId)

