#!/usr/bin/env python
############################################################################
#  
#  Copyright (c)2000 FourThought, Inc.
#  http://ftsuite.com/4ODS
#
############################################################################
#
# File Name: 		4odb_destroy.py
#
# Documentation:	http://docs.ftsuite.com/4ODS/Tools/4odb_destroy.py.html
#


#Dump a 4ODS database using OIF

import sys

def ErrorExit(program, msg):
    sys.stderr.write("%s: %s\n\n" % (program, msg))
    PrintUsage(program)
    sys.exit(1)

def PrintUsage(program):
    import Ft
    print '4ODS version %s' % Ft.__version__
    print
    print 'Dump a 4ODS database to OIF'
    print
    print 'Usage: %s [options] <database name>' % program
    print 'Options:'
    print '  -h, --help           Display this information'
    print '  -o, --output <file>  Put output into <file>'
    return

def Run(argv):
    import sys, getopt
    from Ft.Ods import StorageManager
    from Ft.Ods.StorageManager import OifAdapter
    from Ft.Ods import Database

    program_name = argv[0]

    options, args = getopt.getopt(argv[1:], 'ho:', ['help','output='])

    if not args:
        ErrorExit(program_name, 'A 4ODS database name required')

    stream = sys.stdout
    for (opt, val) in options:
        if opt in ['-o', '--output']:
            stream = open(val, 'w')
        elif opt in ['-h', '--help']:
            PrintUsage(program_name)
            sys.exit(0)
        else:
            ErrorExit(program_name, 'Unrecognized option: %s' % opt)

    database_name = args[0]
    

    oif = OifAdapter.OifAdapter(database_name)
    oif.begin()
    oif.odmsdump(stream)
    # Just to be safe, although nothing should have changed
    oif.abort()
    sys.exit(0)


if __name__ == '__main__':
    import sys
    Run(sys.argv)
