########################################################################
#
# File Name:            Dbm.py
#
# Documentation:        http://docs.4suite.com/4Rdf/Drivers/Dbm.py.html
#
"""
A simple persistent back end
WWW: http://4suite.com/4RDF         e-mail: support@4suite.com

Copyright (c) 1999,2000 Fourthought Inc, USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

from Ft.Rdf.Drivers import Memory
from Ft.Lib import DbmDatabase

# Management functions
def ExistsDb(dbName):
    name = 'ftrdf__%s' % dbName
    return name in DbmDatabase.GetAllDatabaseNames()

def CreateDb(dbName):
    db = DbmDatabase.CreateDatabase('ftrdf__%s' % dbName)
    db.createTable('bindings')
    db.createTable('acl')
    statements = db.createTable('statements')
    statements['unsorted'] = []
    db.commit()
    return DbAdapter(dbName)

def GetDb(dbName):
    return DbAdapter(dbName)

def DestroyDb(dbName):
    name = 'ftrdf__%s' % dbName
    DbmDatabase.DropDatabase(name)

# The RDF Adapter interface
class DbAdapter(Memory.DbAdapter):
    def __init__(self, name):
        self._name = 'ftrdf__%s' % name
        self._dbm = None

    def begin(self):
        self._dbm = DbmDatabase.Database(self._name)
        self._statements = self._dbm.getTable('statements')['unsorted']
        self._bound = self._dbm.getTable('bindings')
        self._acl = self._dbm.getTable('acl')
        return

    def commit(self):
        self._dbm.getTable('statements')['unsorted'] = self._statements
        self._dbm.commit()
        return

    def rollback(self):
        self._dbm.rollback()
        return
