########################################################################
#
# File Name:            4xslt.py
#
# Documentation:        http://docs.4suite.com/4XSLT/4xslt.py.html
#
"""
Command-line invokation of the 4RDF processor
WWW: http://4suite.com/4XSLT        e-mail: support@4suite.com

Copyright (c) 2001 Fourthought, Inc., USA.   All Rights Reserved.
See  http://4suite.com/COPYRIGHT  for license and copyright information
"""

import os, sys, getopt
from Ft.Lib import pDomlette
from Ft.Rdf import Model
from Ft.Rdf.Serializers.Dom import Serializer


g_usage = """
4RDF version %s

Usage: %s [options] <source uri>...
Options:
  -d                Dump the resulting triples to standard output (no presistent changes are made)
  --driver=<name>   Use the given 4RDF backend driver
  --dbname=<name>   Update the RDF model database with the given name 

Note: if you use "-" as the name of the source document, the source will
instead be read from standard input.
"""

def ParseCommandLine(argv):
    #FIXME: Add an option to create a DB, and one to set model URI
    dump = 0
    driver = ""
    dbname = ""
    command_line_error = 0
    trace_on_error = 0
    source = ""
    try:
        optlist, args = getopt.getopt(argv[1:], 'd', ['driver=', 'dbname='])
        source = args[0]
        for k, v in optlist:
            if k == "-d":
                dump = 1
            elif k == "--driver":
                driver = v
            elif k == "--dbname":
                dbname = v
            else:
                command_line_error = 1
    except:
        command_line_error = 1

    if command_line_error:
        import Ft
        print g_usage % (Ft.__version__, os.path.basename(argv[0]))
        sys.exit(1)

    return (dump, driver, dbname, source,command_line_error)


def Run(argv):
    (dump, driver, dbname, source, command_line_error) = ParseCommandLine(argv)
    reader = pDomlette.PyExpatReader()
    if dump:
        driver = "Memory"

    driver = "Ft.Rdf.Drivers." + driver
    driver_mod = __import__(driver, {}, {}, ["*"])

    if dump:
        db = driver_mod.CreateDb(dbname)
    else:
        db = driver_mod.GetDb(dbname)

    m = Model.Model(db)
    if source == '-':
        doc = reader.fromStream(sys.stdin)
    else:
        doc = reader.fromUri(source)

    serializer = Serializer()
    serializer.deserialize(m, doc, '')

    if dump:
        stmts = m.complete(None, None, None)
        print 'The following is a list of remaining tuples, each in the form "subject, predicate, object".'
        print "["
        for stmt in stmts:
            print '("%s", "%s", "%s"),'%(stmt.subject, stmt.predicate, stmt.object)
        print "]"
            
    reader.releaseNode(doc)
    return


if __name__ == '__main__':
    Run(sys.argv)

